# -*- coding:utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sqlite3
import sys
try:
    reload(sys)
    sys.setdefaultencoding('utf8')
except Exception as e:
    pass
import os
import logging

class AppUpdateTypeDB():
    def __init__(self, app_update_type_path=""):
        print(app_update_type_path)
        if app_update_type_path == "":
            self.app_update_type_path = sys.path[0]+'/app_update_type.db'
        else:
            self.app_update_type_path = app_update_type_path
        self.create()

    def create(self):
        table_new = """CREATE TABLE app_update_type (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        debname      varchar(100) NOT NULL,
                        appname      varchar(200) DEFAULT NULL,
                        update_type  varchar(20) DEFAULT NULL,
                        process_name varchar(200) DEFAULT NULL)"""
        if os.path.exists(self.app_update_type_path):
            return

        conn = None
        try:
            dir_name = os.path.dirname(self.app_update_type_path)
            if os.path.isfile(dir_name):
                os.remove(dir_name)
            if not os.path.isdir(dir_name):
                os.makedirs(dir_name)
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            cur.execute(table_new)
            conn.commit()
        except Exception as e:
            logging.error( "create app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()

    def search(self):
        ret = []
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                onerecord = {}
                onerecord['id'] = value[0]
                onerecord['debname'] = value[1]
                onerecord['appname'] = value[2]
                onerecord['update_type'] = value[3]
                onerecord['process_name'] = value[4]
                ret.append(onerecord)
        except Exception as e:
            logging.error( "search app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def get_all_debnames(self):
        ret = []
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                ret.append(value[1])
        except Exception as e:
            logging.error( "get_all_debnames app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def get_debname_with_id_and_type(self):
        ret = {}
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                ret[value[1]] = {}
                ret[value[1]]['id'] = value[0]
                ret[value[1]]['update_type'] = value[3]
        except Exception as e:
            logging.error( "get_debname_with_id_and_type app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def get_debs_msg(self):
        ret = {}
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                ret[value[1]] = {}
                ret[value[1]]['id'] = value[0]
                ret[value[1]]['update_type'] = value[3]
                ret[value[1]]['process_name'] = value[4]
        except Exception as e:
            logging.error( "get_debs_msg app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret


    def get_debname_with_type_and_process_name(self):
        ret = {}
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                ret[value[1]] = {}
                ret[value[1]]['update_type'] = value[3]
                ret[value[1]]['process_name'] = value[4]
        except Exception as e:
            logging.error( "get_debname_with_type_and_process_name app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def get_debname_with_type(self):
        ret = {}
        conn = None
        try:
            command = "SELECT * FROM app_update_type"
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                ret[value[1]] = value[3]
        except Exception as e:
            logging.error( "get_debname_with_type app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def replace(self, sql_id, debname='', appname='', update_type='', process_name=''):
        conn = None
        try:
            command = "REPLACE INTO app_update_type (id, debname, appname, update_type, process_name) "\
                    "VALUES (%d, '%s', '%s', '%s', '%s')" % (sql_id, debname, appname, update_type, process_name)
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            conn.commit()
        except Exception as e:
            logging.error( "replace app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()

    def insert(self, debname, appname='', update_type='', process_name=''):
        try:
            debname_with_id_json = self.get_debname_with_id_and_type()
            if debname in debname_with_id_json:
                deb_id = debname_with_id_json[debname]["id"]
                self.replace(deb_id, debname, appname, update_type, process_name)
                return
        except Exception as e:
            logging.error( "insert app_update_type database " + str(e) )

        try:
            command = "INSERT INTO app_update_type (debname, appname, update_type, process_name) "\
                    "VALUES ('%s', '%s', '%s', '%s')" % ( debname, appname, update_type, process_name)
            conn = sqlite3.connect(self.app_update_type_path)
            cur = conn.cursor()
            cur.execute(command)
            conn.commit()        
        except Exception as e:
            logging.error( "insert app_update_type database " + str(e) )
        finally:
            if conn is not None:
                conn.close()

if __name__ == '__main__':
    home_dir = os.getenv("HOME")
    app_update_type_db_obj = AppUpdateTypeDB(os.path.join(home_dir, ".config/cdos-update/app_update_type.db"))
    app_update_type_db_obj.insert("vim1", "aa", "aaa1", "process_name1")
    app_update_type_db_obj.insert("vim2", "aa", "aaa1", "process_name1")
    # app_update_type_db_obj.replace( 1, "vim1", "aaa2", "aaa3", "")
    print(app_update_type_db_obj.get_all_debnames())
    print(app_update_type_db_obj.get_debname_with_id_and_type())
    print(app_update_type_db_obj.get_debname_with_type())
    for info in app_update_type_db_obj.search():
        print(info)


