#! /bin/python3
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
os.environ['GI_TYPELIB_PATH'] = 'libcdosnotify:' + os.environ.get('GI_TYPELIB_PATH', '')

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
import apt

import sys
import traceback
import tempfile
import _thread
import threading
import logging
import time
import json

from datetime import datetime
from datetime import timedelta
from threading import Timer

from patch_lib import Result
from patch_lib import determineCheck
from patch_lib import getSleeptimeBeforeNextCheck
from patch_lib import setLastCheckTime

from app_update_type import AppUpdateTypeDB

from dbus_updates import License_dbusInterface

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename=tempfile.mktemp("cdos-back-update.log"),level=logging.DEBUG)

global uidir
global timer_id
uidir = sys.path[0] + "/ui"
timer_id = -1

global dbus_object
dbus_object = None

global autoinstall, autodownload, willcheckupdate, dayfreq, hourfreq,check_last_time,install_last_time
willcheckupdate = True
dayfreq = 0
hourfreq = 0

global last_update_cache_time
last_update_cache_time = None

global update_info
update_info = {}

global appupdatemode
appupdatemode = "DEFAULT_UPDATE"

global app_update_type_db_obj
app_update_type_db_obj = AppUpdateTypeDB(os.path.join(os.getenv("HOME"), ".config/cdos-update/app_update_type.db"))

global app_update_type_json
app_update_type_json = app_update_type_db_obj.get_debname_with_type_and_process_name()

global env
env = {}
env["HOME"] = os.getenv("HOME")
env["USER"] = os.getenv("USER")
env["DISPLAY"] = os.getenv("DISPLAY")
env["PATH"] = os.getenv("PATH")

global patch_name
global patch_id

global schema
schema = Gio.Settings(schema = "org.cdos.update")

global iface_dbus_license

global download_work
download_work=[]

global list_notify
list_notify = []

import gettext as gettext1
gettext1.bindtextdomain('cdosupdate', '/usr/share/locale')
gettext1.textdomain('cdosupdate')
_ = gettext1.gettext

path_lib = os.popen("dpkg -L nfs-ui-sdk | grep libUiSDK.so").read()
files_lib = path_lib.split("\n")

name_lib = ""
file_ui_sdk = ""

for lib_p in files_lib:
    lib_f = lib_p.split("/")
    lib_f = lib_f[len(lib_f) - 1]
    if lib_f.startswith("libUiSDK.so"):
        index_s = len("libUiSDK.so")
        if len(name_lib) == 0:
            name_lib = lib_f[index_s:]
            file_ui_sdk = lib_p
        elif name_lib < lib_f[index_s:]:
            name_lib = lib_f[index_s:]
            file_ui_sdk = lib_p

from ctypes import *
lib_ui_sdk = cdll.LoadLibrary(file_ui_sdk)
CFUN_DEF = CFUNCTYPE(None)

lock = threading.Lock()

class nofity_send(threading.Thread):
    def __init__(self, title, msg, iconPath, msgConfirm, msgCancel, funConfirm, funCancel):
        threading.Thread.__init__(self)
        self.threadID = str(len(list_notify))
        self.title = title
        self.msg = msg
        self.iconPath = iconPath
        self.soundPath = ""
        self.level = "normal"
        if msgConfirm:
            self.msgConfirm = msgConfirm.encode()
        else:
            self.msgConfirm = None
        if msgCancel:
            self.msgCancel = msgCancel.encode()
        else:
            self.msgCancel = None
        if funConfirm:
            self.funConfirm = CFUN_DEF(funConfirm)
        else:
            self.funConfirm = None
        if funCancel:
            self.funCancel = CFUN_DEF(funCancel)
        else:
            self.funCancel = None
    def run(self):
        lock.acquire()
        lib_ui_sdk.us_nofity_send(self.title.encode(), self.msg.encode(), self.iconPath.encode(), self.soundPath.encode(), self.level.encode(), self.msgConfirm, self.msgCancel, self.funConfirm, self.funCancel)
        lock.release()

# def nofity_send(title, msg, iconPath, msgConfirm, msgCancel, funConfirm, funCancel):
#     soundPath = ""
#     level = "critical"
#     t = threading.Thread(target=lib_ui_sdk.us_nofity_send, args=(title.encode(), msg.encode(), iconPath.encode(), soundPath.encode(), level.encode(), msgConfirm.encode(), msgCancel.encode(), CFUN_DEF(funConfirm), CFUN_DEF(funCancel),))
#     t.start()
#     return t

class PatchStatus:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

def handle_upgradeStatusChanged(statusCode, status_text, paramList):
    logging.info("handle_upgradeStatusChanged")
    def dist_upgrade(upgrade_cmd):
        logging.info( upgrade_cmd)
        os.system("bash " + upgrade_cmd)

    if statusCode == 100:
        try:
            upgrade_tool_main_path = os.path.join(paramList["upgrade_tool_dir"], "distupgrade")
            thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
            time.sleep(1)
        except Exception as e:
            logging.error("handle_upgradeStatusChanged Exception:" + str(e))
        finally:
            pass

def activate_now_call():
    logging.info("activate_now_call")
    iface_dbus_license.showLicenseActivator()

#弹出系统更新主页面.
def enter_update_call():
    logging.info("enter_update_call")
    # iface_dbus_license = License_dbusInterface()
    licenseState = iface_dbus_license.getLicenseState()
    if licenseState == 2 or licenseState == 4: #激活状态为授权已过期和试用期已过期，弹出激活窗口，并退出
        notify = nofity_send(_('NFS Desktop OS Update'), _('Your system has not been activated yet. Please activate it as soon as possible to avoid affecting the complete system functionality and operating experience.'),
                            "cdos-update", _('Activate Now'), None,
                            activate_now_call, None)

        notify.start()
        list_notify.append(notify)
    else:
        os.system("/usr/bin/python3 /usr/share/cdos-update/frontend/check_updates.py &")

def restart_notify_call():
    logging.info("restart_notify_call")
    global dbus_object
    dbus_object.sys_reboot()
    

def cancel_notify_call():
    logging.info("cancel_notify_call")

def checkmainwindow():
    try:
        session_bus = dbus.SessionBus()
        remote_object_2 = session_bus.get_object("org.cdos.update.ui", "/org/cdos/update/ui" )
        client_iface = dbus.Interface(remote_object_2, "org.cdos.update.ui")
        if client_iface.show() == True:
            return True
        return False
    except Exception as e:
        return False

def get_pid_and_process_name():
    pid_process_json = {}
    for dir_name in os.listdir("/proc"):
        cmdline_file = os.path.join( os.path.join("/proc", dir_name), "cmdline")
        if dir_name.isdigit() and os.path.isfile(cmdline_file):
            try:
                pid_cmd_fd = open( cmdline_file)
                pid_cmd = pid_cmd_fd.read()
                pid_cmd_fd.close()
                if pid_cmd.endswith("\x00"):
                    pid_cmd = pid_cmd[:-1]
                pid_process_json[pid_cmd] = dir_name
            except Exception as e:
                continue
    return pid_process_json

def updateapps(update_info, app_update_type_json):
    dialogui_show_flag = False
    system_bus = dbus.SystemBus()
    daemon_remote_object = system_bus.get_object("org.cdos.update", "/org/cdos/update" )
    daemon_iface = dbus.Interface(daemon_remote_object, "org.cdos.update")
    pid_process_json = get_pid_and_process_name()
    for debname in app_update_type_json:
        if ( not debname in update_info) and ( not debname in update_info["_deb_res"]["list"]):
            continue
        if app_update_type_json[debname]["update_type"] == "AUTO_UPDATE":
            print ("update debname:" + debname)
            metadata = {}
            metadata["package_name"] = debname
            metadata["version"] = ""
            metadata["pid"] = ""
            metadata["process_name"] = ""
            metadata["env_home"] = os.getenv("HOME")
            metadata["env_path"] = os.getenv("PATH")
            metadata["env_user"] = os.getenv("USER")
            metadata["env_display"] = os.getenv("DISPLAY")
            if False:
                metadata["exit_flag"] = False
            else:
                process_name = app_update_type_json[debname]["process_name"]
                if process_name in pid_process_json:
                    metadata["pid"] = pid_process_json[process_name]
                    metadata["process_name"] = process_name
                    metadata["exit_flag"] = True
                else:
                    metadata["exit_flag"] = False
            daemon_iface.installAppUpdate(metadata)

        if app_update_type_json[debname]["update_type"] == "AUTO_PROMPT_UPDATE":
            print ("AUTO_PROMPT_UPDATE")
            dialogui_show_flag = True
    if dialogui_show_flag:
        notify = nofity_send(_('NFS Desktop OS Update'), _('Download and install app updates?'), "cdos-update", _('yes'), None, enter_update_call, None)
        notify.start()
        list_notify.append(notify)

#桌面通知，弹出通知弹框，询问用户是否更新.
def refreshUpdateThread():
    logging.debug("refreshUpdateThread")
    global dbus_object
    global update_info
    global app_update_type_json
    global patch_name
    global patch_id
    global download_work
    
    canBeUpdate = False
    appCanBeUpdate = False
    silentUpdate = False

    if checkIdleTime() == False:
        return

    try:
        # repo_url  = "http://172.30.13.37/"
        # site_path = repo_url + "patches/"
        # getpatchlistinfo_site_patch = site_path + "getpatchesinfoWithSystemVersion"
        # upgrade_path = repo_url + "data/cdos-update/cdos-upgrade-new-server/meta-release"
        #repo_url = "http://update.os.nfschina.com/static"

        new_repo_url = "https://repos.os.nfschina.com"
        patch_url = new_repo_url + "/patch-server/patch_update.json"

        update_info = dbus_object.getAllUpdates(patch_url)
        if ("_patches" in update_info):
            if  ("latestPatchName" in update_info["_patches"]) and (update_info["_patches"]["latestPatchName"] != "") and ("patchID" in update_info["_patches"]) and ("status" in update_info["_patches"]):
                canBeUpdate = True
                download_work.append(update_info["_patches"]["latestPatchName"])
        if ("_secure_patches" in update_info):
            if  ("latestPatchName" in update_info["_secure_patches"]) and (update_info["_secure_patches"]["latestPatchName"] != "") and ("patchID" in update_info["_secure_patches"]) and ("status" in update_info["_secure_patches"]):
                canBeUpdate = True
                download_work.append(update_info["_secure_patches"]["latestPatchName"])
        if ("_silence_patches" in update_info):
            if  ("latestPatchName" in update_info["_silence_patches"]) and ("patchID" in update_info["_silence_patches"]) and ("status" in update_info["_silence_patches"]):
                silentUpdate = True


        # repo_url  = "http://update.os.nfschina.com"
        # site_path = repo_url + ":2367/"
        # getpatchlistinfo_site_patch = site_path + "getpatchesinfoWithSystemVersion"
        # upgrade_path = repo_url + ":8080/data/upgrade/cdos-upgrade/"

        # update_info = dbus_object.getAllUpdates(os.environ["HOME"] ,upgrade_path, getpatchlistinfo_site_patch)  
        # if update_info["_upgrade"]["status"] == True:
        #     canBeUpdate = True
        # elif len(update_info["_patches"]) > 0:
        #     canBeUpdate = True
        # elif update_info["_deb_res"]["num"] > 0:
        #     canBeUpdate = True
        # elif len(update_info) > 3:
        #     canBeUpdate = True

        # if len(update_info) > 3:
        #     appCanBeUpdate = True
    except Exception as e:
        logging.error("refreshUpdateThread Exception:" + str(e))
        return

    # 设置checktime
    #setLastCheckTime()
    if silentUpdate:
        if checkmainwindow():
            return
        logging.info("will do silentUpdate")
        setLastCheckTime()

        licenseState = iface_dbus_license.getLicenseState()
        if licenseState == 2 or licenseState == 4:#激活状态为授权已过期和试用期已过期，不进行静默更新
            return
        #update_info#
        patch_name = update_info["_silence_patches"]["latestPatchName"]
        patch_id = update_info["_silence_patches"]["patchID"]
        dbus_object.download_patch(env, patch_name)
        #dbus_object.install_patch(env, patch_name, patch_id)
        notify = nofity_send(_('NFS Desktop OS Update'), _('the system is silently updating'), "cdos-update", None, None, None, None)
        notify.start()
        #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('the system is silently updating'), 'cdos-update')
        #notify.show()
        list_notify.append(notify)
        return
        
    if canBeUpdate:
        # if mainwindow is show
        if checkmainwindow():
            return
            
        logging.info("will do sys or secure update, the download_work:" + str(download_work))
        setLastCheckTime()
        
        if schema.get_int("distupgrade-opts"):##手动下载更新
            notify = nofity_send(_('NFS Desktop OS Update'), _('Check that your system has updates available!'), "cdos-update", _('Look'), None, enter_update_call, None)
            notify.start()
            list_notify.append(notify)
        else:                                   #自动下载更新
            if len(download_work) != 0:
                download_patch_name = download_work.pop(0)
                logging.info("will download " + download_patch_name)
                dbus_object.download_patch(env, download_patch_name)
            else:
                logging.error("download_work is empty")

        return


global bListenUpdateCache
bListenUpdateCache = False
def handle_cachestatusChanged(msg):
    logging.info( "handle_cachestatusChanged:" + str(msg))
    if willcheckupdate:
        global bListenUpdateCache
        global dbus_object
        if msg['status'] == 'success':
            global last_update_cache_time
            last_update_cache_time = datetime.now()
        if bListenUpdateCache:
            #dbus_object.update_software_database()#下载新的软件中心数据库，注释掉.
            if msg['status'] == 'success':
                GLib.idle_add(refreshUpdateThread)
            if msg['status'] == 'failed':
                GLib.idle_add(refreshUpdateThread)
            bListenUpdateCache = False

def handle_statusChanged(statusCode, status_text, patchID, correct_install_patches_list, patchList, progress, paramList):
    if statusCode == PatchStatus.ALLINSTALLED:
        if ('reboot' in paramList) and (paramList['reboot'] == True):
            notify = nofity_send(_('NFS Desktop OS Update'), _('Some updates will take effect after reboot OS'), "cdos-update", None, None, None, None)
            notify.start()
            #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('Some updates will take effect after reboot OS'), 'cdos-update')
            #notify.show()
            list_notify.append(notify)
            pass

def handle_backup_recover_status_changed(msg):
    logging.info("handle_backup_recover_status_changed:" + str(msg))
    if not ("type" in msg) or checkmainwindow():
        return

    if msg['action'] == 'backup':
        if msg['status'] == 'inprogress':
            logging.info("silence_patch backup in progress")
        elif msg['status'] == 'success':
            logging.info("silence_patch backup success")
        elif msg['status'] == 'failed':
            logging.info("silence_patch back up failed")

    elif msg['action'] == 'recover':
        if msg['status'] == 'inprogress':
            logging.info("silence_patch restoring")
        elif msg['status'] == 'success':
            logging.info("silence_patch recover success")
            notify = nofity_send(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), "cdos-update", _('Restart'), None, restart_notify_call, None)
            notify.start()
            #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), 'cdos-update')
            #notify.add_action('yes', _('Restart'), restart_notify_call, None, None)
            #notify.show()
            list_notify.append(notify)
        elif msg['status'] == 'failed':
            logging.error("silence_patch recover failed")
            notify = nofity_send(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), "cdos-update", _('Restart'), None, restart_notify_call, None)
            notify.start()
            #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), 'cdos-update')
            #notify.add_action('yes', _('Restart'), restart_notify_call, None, None)
            #notify.show()
            list_notify.append(notify)

def handle_patchStatusChanged(msg):
    logging.info("handle_patchStatusChanged:" + str(msg))
    if not ("type" in msg) or checkmainwindow():
        return
        
    global download_work
    if msg['type'] == 'silence_patch' and msg['action'] == 'download':
        if msg['status'] == 'inprogress':
            logging.info("silence_patch downloaded" + str(int(msg['percent'])) + "%")
        elif msg['status'] == 'success':
            logging.info("silence_patchdownloaded" + str(int(msg['percent'])) + "%")
            dbus_object.backup()
            #if bool(dbus_object.backup()) == False:
            dbus_object.install_patch(env, patch_name, patch_id)
        elif msg['status'] == 'failed':
            logging.info("silence_patch downloaded" + str(int(msg['percent'])) + "%")

    elif msg['type'] == 'silence_patch' and msg['action'] == 'install':
        if msg['status'] == 'inprogress':
            logging.info("silence_patch install" + str(int(msg['percent'])) + "%")
        elif msg['status'] == 'success':
            logging.info("silence_patch install success")
            notify = nofity_send(_('NFS Desktop OS Update'), _('the system has completed an important update. it is recommended that you restart your computer immediately'), "cdos-update", _('Restart'), None, restart_notify_call, None)
            notify.start()
            #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('the system has completed an important update. it is recommended that you restart your computer immediately'), 'cdos-update')
            #notify.add_action('yes', _('Restart'), restart_notify_call, None, None)
            #notify.add_action('no', _('no'), cancel_notify_call, None, None)
            #notify.show()
            list_notify.append(notify)
        elif msg['status'] == 'failed':
            logging.error("silence_patch install failed")
            if bool(dbus_object.recover()) == False:
                notify = nofity_send(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), "cdos-update", _('Restart'), None, restart_notify_call, None)
                notify.start()
                #notify = CdosNotify.Notification.new(_('NFS Desktop OS Update'), _('failed to install update, please restart your computer!'), 'cdos-update')
                #notify.add_action('yes', _('Restart'), restart_notify_call, None, None)
                #notify.show()
                list_notify.append(notify)
    
    elif msg['action'] == 'download':
        if msg['status'] == 'inprogress':
            logging.info(str(msg['type']) + " downloaded" + str(int(msg['percent'])) + "%")
        elif msg['status'] == 'success':
            logging.info(str(msg['type']) + "downloaded success")
            if len(download_work) != 0:
                download_patch_name = download_work.pop(0)
                logging.info("will download " + download_patch_name)
                dbus_object.download_patch(env, download_patch_name)
            else:
                notify = nofity_send(_('NFS Desktop OS Update'), _('Check that your system has updates available!'), "cdos-update", _('Look'), None, enter_update_call, None)
                notify.start()
                dbus_object.backup()
                list_notify.append(notify)
        elif msg['status'] == 'failed':
            logging.error(str(msg['type']) + "downloaded failed")
            download_work = []

def refreshUpdate(isMainWindow=False):
    global bListenUpdateCache
    global dbus_object
    if checkmainwindow() or checkIdleTime() == False:
        return
    #env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
    bListenUpdateCache = True
    dbus_object.update_cache(env)

def update_timer():
    global timer_id
    logging.debug('update_timer')

    determine_res = determineCheck()
    if timer_id > -1:
        GLib.source_remove(timer_id)
        timer_id = -1
    # 出错
    if determine_res.getError() != None:
        timer_id = GLib.timeout_add_seconds(60, update_timer)
        return Result(determine_res.getError(), None)
    # 不需要进行补丁升级检查
    if determine_res.getValue() == False:
    # if False:
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
        return Result("it's not time for patch", None)
    # 需要进行补丁升级检查
    else:
        # 检查是否需要更新
        # mainCheckWindow.set_last_checkTime(check_last_time)
        refreshUpdate()
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
    return 1

#调用org.cdos.update的dbus接口update_cache，更新源的cache.
def refreshCache():
    # env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
    # dbus_object.update_cache(env)
    # one_hour = Timer(100, refreshCache)
    # one_hour.start()
    # return
    global last_update_cache_time
    if last_update_cache_time is None:
        if not checkmainwindow() and checkIdleTime():
            #env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
            dbus_object.update_cache(env)
        one_hour = Timer(3600, refreshCache)
        one_hour.start()
    else:
        nowtime = datetime.now()
        unchecktime = nowtime - last_update_cache_time
        one_day = timedelta(hours=24)
        nexttime = one_day - unchecktime
        if nexttime.total_seconds() > 0:
            nextcheck = Timer(nexttime.total_seconds(), refreshCache)
            nextcheck.start()
        else:
            if not checkmainwindow() and checkIdleTime():
                #env = { 'HOME': os.environ['HOME'], 'USER': os.environ['USER'], 'DISPLAY':os.environ['DISPLAY'] }
                dbus_object.update_cache(env)
            one_hour = Timer(3600, refreshCache)
            one_hour.start()

def checkIdleTime():
    #当前仅认为软件中心正在运行时是非idle态
    if os.system("ps -ef | grep -E ' cdos-softcenter| nfs-appstore| cdos-appstore' | grep -v grep") == 0:
        return False
    else:
        return True

global client_dbus_name
global client_dbus_path
global client_dbus_iface
client_dbus_name = "org.cdos.update.client"
client_dbus_path = "/org/cdos/update/client"
client_dbus_iface =  "org.cdos.update.client"

class CDOSUpdateClientDaemon(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(client_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, client_dbus_path)
        self.cache=apt.Cache()

    #设置系统更新模式，cdos-api中的应用更新api使用.
    @dbus.service.method(client_dbus_iface, in_signature='sss', out_signature='bs')
    def setAppUpdateMode(self, pkg_name, update_mode, process_name):
        self.cache.open()
        if not pkg_name in self.cache:
            return False, pkg_name + " not in cache"
        if update_mode == "AUTO_UPDATE" or update_mode == "MANUAL_UPDATE" \
            or update_mode == "AUTO_PROMPT_UPDATE" or update_mode == "DEFAULT_UPDATE":
            try:
                # schema = Gio.Settings(schema = "org.cdos.update")
                # schema.set_string("appupdatemode", update_mode)
                global app_update_type_json
                global app_update_type_db_obj
                app_name = ""
                app_update_type_db_obj.insert(pkg_name, app_name, update_mode, process_name)
                app_update_type_json[pkg_name] = {}
                app_update_type_json[pkg_name]["update_type"] = update_mode
                app_update_type_json[pkg_name]["process_name"] = process_name
                print (app_update_type_json)
                return True, ""
            except Exception as e:
                print (str(e))
                return False, str(e)
        return False, "update mode is incorrect"

    #获取系统更新模式，cdos-api中的应用更新api使用.
    @dbus.service.method(client_dbus_iface, in_signature='s', out_signature='s')
    def getAppUpdateMode(self, pkg_name):
        self.cache.open()
        if not pkg_name in self.cache:
            return "DEFAULT_UPDATE"
        try:
            global app_update_type_json
            if pkg_name in app_update_type_json:
                return app_update_type_json[pkg_name]["update_type"]
            else:
                return "DEFAULT_UPDATE"
        except Exception as e:
            return "DEFAULT_UPDATE"

def getRegisterStatus():
    try:
        system_bus = dbus.SystemBus()
        registerObj = system_bus.get_object('com.nfs.register', '/')
        iface = dbus.Interface(registerObj, dbus_interface='com.nfs.register')
        registerInfo = iface.get_dbus_method("getRegisterStatus", dbus_interface=None)
        return registerInfo('os', '')
    except Exception as e:
        logging.error("getRegisterStatus:" + str(e))
        return None

if __name__ == "__main__":

    try:
        dbml = DBusGMainLoop(set_as_default=True)
        bus = dbus.SystemBus()
        remote_object = bus.get_object("org.cdos.update", "/org/cdos/update")
        dbus_object = dbus.Interface(remote_object, 'org.cdos.update')

        dbus.set_default_main_loop(dbml)
    except Exception as e:
        logging.error("Error Happened, please check the dbus service org.cdos.update has started")
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        sys.exit(1)  #进程退出

    #检查系统是否注册或者注册是否过期
    #registerResult = getRegisterStatus()
    #if registerResult == None or registerResult[0] == 0 or (registerResult[0] == 1 and registerResult[1] <= 0):
    #    logging.info("Register status error.")
    #    sys.exit(0)
    iface_dbus_license = License_dbusInterface()
    willcheckupdate = schema.get_boolean("checkupdate")#定义是否检测更新？
    
    def on_willcheckupdate_changed(self, c):
        global willcheckupdate
        willcheckupdate = schema.get_boolean("checkupdate")

    schema.connect("changed::checkupdate", on_willcheckupdate_changed)
    
    try:
        bus.add_signal_receiver(handle_cachestatusChanged,'cacheStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_upgradeStatusChanged,'upgradeStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_statusChanged,'statusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_patchStatusChanged,'patchStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        bus.add_signal_receiver(handle_backup_recover_status_changed,'backup_recover_status_changed', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        
    except Exception as e:
        logging.error(e)
        sys.exit(1)  #进程退出

    def on_dayfreq_changed(c):
        global dayfreq
        global timer_id
        dayfreq = schema.get_int("check-day")
        # mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_dayfreq_changed(self, c):
        GLib.idle_add(on_dayfreq_changed, c)

    def on_hourfreq_changed(c):
        global hourfreq
        global timer_id
        hourfreq = schema.get_int("check-hour")
        # mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_hourfreq_changed(self, c):
        GLib.idle_add(on_hourfreq_changed, c)

    dayfreq = schema.get_int("check-day")
    hourfreq = schema.get_int("check-hour")
    schema.connect("changed::check-day", idle_on_dayfreq_changed)
    schema.connect("changed::check-hour", idle_on_hourfreq_changed)

    CDOSUpdateClientDaemon()#用于cdos-api中的应用更新api.
    one_hour = Timer(3600, refreshCache)
    one_hour.start()
    timer_id = GLib.timeout_add_seconds(3600, update_timer)
    Gtk.main()

