#!/usr/bin/python3
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sys
import dbus
import time
import os

from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from dbus.mainloop.glib import DBusGMainLoop
from check_upgrade import *

ISOTIMEFORMAT = '%Y-%m-%d %H:%M'


class UICheck(object):
    
    def __init__(self, uidir):
        import gettext as gettext1
        gettext1.bindtextdomain('cdosupdate', "/usr/share/locale/")
        gettext1.textdomain('cdosupdate')
        self._ = gettext1.gettext

        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\
            *:selected {
                border: none;
                background: #b6ddf6;
                box-shadow: none;
                color: #333333;
            }
            *:hover {
                border: none;
                background: #e3e3e3;
                color: #333333;
            }
            #menu-viewport {
                background:#F3F3F3;
            }
            #eventbox18,
            #note-viewport {
                background:#FCFCFC;
            }
            #label95 {
                color:#333333;
                font-size:20px;
            }
            #label96,
            #label98,
            #label99 {
                color:#333333;
                font-size:13px;
            }
            #upgrade_title,
            #update_title {
                 color: #333333;
                 font-size: 20px;
                 font-weight: 500;
            }
            #upgrade_tip,
            #status_text,
            #status_detail_text,
            #label3,
            #label4,
            #label5,
            #label6,
            #label7,
            #label8,
            #label16,
            #label17,
            #label18,
            #label19,
            #label20,
            #label21 {
                 color: #333333;
                 font-size: 13px;
            }

            #enterbutton,
            #enterbutton1,
            #confirm,
            #cancel {
                -GtkWidget-focus-padding: 0;
                -GtkWidget-focus-line-width: 0;
                -GtkButton-inner-border: 0;

                background: #44A6E5;
                color: #FFFFFF;
                box-shadow: none;
                font-weight: normal;
                font-size: 13px;
                border: none;
                border-radius: 0;
                border-color: #44A6E4;
                border-width: 1px;
                border-style: solid;
            }


            #enterbutton:active,
            #enterbutton:active:hover,
            #enterbutton1:active,
            #enterbutton1:active:hover,
            #confirm:active,
            #confirm:active:hover,
            #cancel:active,
            #cancel:active:hover {
                background: #3C93D3;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #enterbutton:hover,
            #enterbutton1:hover,
            #confirm:hover,
            #cancel:hover {
                background: #48B2ED;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #enterbutton:focus,
            #enterbutton:hover:focus,
            #enterbutton:active:focus,
            #enterbutton1:focus,
            #enterbutton1:hover:focus,
            #enterbutton1:active:focus,
            #confirm:focus,
            #confirm:hover:focus,
            #confirm:active:focus,
            #cancel:focus,
            #cancel:hover:focus,
            #cancel:active:focus {
                border-color: shade(#44A6E4, 0.8);
                box-shadow: 0px 0px 0px 0px;
            }
            #confirm:insensitive,
            #confirm :insensitive,
            #cancel:insensitive,
            #cancel :insensitive {
                background: #cccccc;
                color: #FFFFFF;
                border: none;
            }

            #checkupgrade, 
            #changesettings,
            #checkupdate {
                -GtkWidget-focus-padding: 0;
                -GtkWidget-focus-line-width: 0;
                -GtkButton-inner-border: 0;

                background: #F3F3F3;
                color: #363636;
                box-shadow: none;
                font-weight: normal;
                font-size: 14px;
                border:1px #6eb5e4;
            }

            #checkupgrade:hover,
            #checkupdate:hover,
            #changesettings:hover {
                background: #e3e3e3;
                border: 1px #6eb5e4;
            }

           #checkupgrade.tab_active_button,
           #checkupdate.tab_active_button,
           #changesettings.tab_active_button {
                background: #b6ddf6;
                color: #333333;
                font-size: 14px;
                border: 1px #6eb5e4;
            }

           #checkupgrade.tab_active_button:hover,
           #checkupdate.tab_active_button:hover,
           #changesettings.tab_active_button:hover {
                font-size: 14px;
                border: 1px #6eb5e4  solid;
            }

            GtkProgressBar {
                padding: 0;
                border-radius: 0;
                border-color: #449AE8;
                background: #FFFFFF;
            }

            .progressbar,
            .progressbar row,
            .progressbar row:hover,
            .progressbar row:selected,
            .progressbar row:selected:focus {
                -GtkButton-inner-border: 0;
                border: none;
                border-radius: 0;
                border-color: #449AE8;
                border-width: 1px;
                border-style: solid;
                background: #449AE8;
            }

            .progressbar.trough,
            .progressbar.trough row,
            .progressbar.trough row:hover,
            .progressbar.trough row:selected,
            .progressbar.trough row:selected:focus {
                -GtkButton-inner-border: 0;
                color: #FFFFFF;
                border: none;
                border-radius: 0;
                background: #FFFFFF;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.builder = Gtk.Builder()
        self.builder.add_from_file(uidir + "/mainwindow.ui")
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("mainwindow")
        self.window.set_title(self._("NFS Desktop OS Update"))

        self.menu_viewport = self.builder.get_object('menu-viewport')
        self.menu_viewport.set_name('menu-viewport')

        self.changesettings =  self.builder.get_object("changesettings")
        self.changesettings.set_name('changesettings')
        self.changesettings.set_label(self._("Change Settings"))
        self.changesettings.connect("clicked", self.on_changesettings_clicked)
        self.checkupdate =  self.builder.get_object("checkupdate")
        self.checkupdate.set_name('checkupdate')
        self.checkupdate.set_label(self._("Check Update"))
        self.checkupdate.connect("clicked", self.on_checkupdate_clicked)
        self.checkupgrade =  self.builder.get_object("checkupgrade")
        self.checkupgrade.set_name('checkupgrade')
        self.checkupgrade.set_label(self._("NFS Desktop OS Upgrade"))
        self.checkupgrade.connect("clicked", self.on_checkupgrade_clicked)        

        self.note_viewport = self.builder.get_object("note-viewport")
        self.note_viewport.set_name("note-viewport")
        self.notepage = self.builder.get_object("notebook_page")
        self.notepage.set_name("notebook_page")
        self.progress_pages = {
            'checkupdate': 0,
            'settings': 1,
            'checkupgrade': 2,
        }
        self.checkupdate.get_style_context().add_class('tab_active_button')
        self.changesettings.get_style_context().remove_class('tab_active_button')
        self.checkupgrade.get_style_context().remove_class('tab_active_button')
        self.notepage.set_current_page(self.progress_pages['checkupdate'])

        self.updatefrequence_viewport = self.builder.get_object("updatefrequence_viewport")
        self.updatefrequence_viewport.set_name("updatefrequence_viewport")
        self.dayfreq_viewport = self.builder.get_object("dayfreq_viewport")
        self.dayfreq_viewport.set_name("dayfreq_viewport")
        self.hourfreq_viewport = self.builder.get_object("hourfreq_viewport")
        self.hourfreq_viewport.set_name("hourfreq_viewport")

        self.updateFrequence_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.updateFrequenceArray = [self._("Install new updates automatically (recommended)"), self._("Download updates but let me choose whether to install updates"), self._("Check for updates but let me choose whether to download and install the updates"), self._("Never check for updates (not recommended)")]
        for item in self.updateFrequenceArray:
            self.updateFrequence_list_store.append([item])
        self.updatefrequence = self.builder.get_object("updatefrequence")
        self.updatefrequence.set_name('updatefrequence')
        # self.updatefrequence.set_wrap_width(10)
        self.updatefrequence.set_model(self.updateFrequence_list_store)
        self.updatefrequence.set_active(0)
        self.updatefrequence.connect("changed", self.on_changed_updatefrequence)

        self.dayfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.dayfreqArray = [self._("Everyday"), self._("Every Monday"), self._("Every Tuesday"), self._("Every Wednesday"), self._("Every Thursday"), self._("Every Friday"), self._("Every Saturday"), self._("Every Sunday")]
        for item in self.dayfreqArray:
            self.dayfreq_list_store.append([item])
        self.dayfreq = self.builder.get_object("dayfreq")
        self.dayfreq.set_name('dayfreq')
        self.dayfreq.set_model(self.dayfreq_list_store)
        self.dayfreq.set_active(0)
        self.dayfreq.connect("changed", self.on_changed_dayfreq)

        self.hourfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        for i in range(24):
            self.hourfreq_list_store.append([str(i)+":00"])
        self.hourfreq = self.builder.get_object("hourfreq")
        self.hourfreq.set_name('hourfreq')
        self.hourfreq.set_model(self.hourfreq_list_store)
        self.hourfreq.set_active(0)
        self.hourfreq.connect("changed", self.on_changed_hourfreq)

        self.status_text =  self.builder.get_object("status_text")
        self.status_text.set_name("status_text")
        self.status_text_image = self.builder.get_object("status_text_image")
        self.status_text_image.set_name("status_text_image")
        # self.status_text_image.set_from_file("img/correct.png")
        # self.status_text_image.set_visible(False)
        self.progress_update =  self.builder.get_object("progress_update")
        self.progress_update.set_name("progress_update")
        self.enterbutton =  self.builder.get_object("enterbutton")
        self.enterbutton.set_name("enterbutton")
        self.lastEnterButtonSignal = None
        self.status_detail_text =  self.builder.get_object("status_detail_text")
        self.status_detail_text.set_name("status_detail_text")

        self.current_check_time_label = self.builder.get_object('label19')
        self.current_check_time_label.set_name("label19")
        self.last_install_time_label = self.builder.get_object('label20')
        self.last_install_time_label.set_name("label20")
        self.label_1 = self.builder.get_object("label1")
        self.label_1.set_text(self._("Check Update"))
        self.label_2 = self.builder.get_object("label2")
        self.label_2.set_text(self._("Change Setting"))
        self.label_14 = self.builder.get_object("update_title")
        self.label_14 .set_text(self._("NFS Desktop OS Update"))
        self.label_14.set_name("update_title")
        self.label_16 = self.builder.get_object("label16")
        self.label_16.set_name("label16")
        self.label_16.set_text(self._("Last check for updates:"))
        self.label_17 = self.builder.get_object("label17")
        self.label_17.set_name("label17")
        self.label_17.set_text(self._("Last time install the update:"))
        self.label_18 = self.builder.get_object("label18")
        self.label_18.set_name("label18")
        self.label_18.set_text(self._("Receive updates:"))
        self.label_21 = self.builder.get_object("label21")
        self.label_21.set_name("label21")
        self.label_21.set_text(self._("Only applies to NFS Desktop OS"))
        self.label_95 = self.builder.get_object("label95")
        self.label_95.set_name("label95")
        self.label_95.set_text(self._("Choose a method to install updates NFS Desktop OS"))
        self.label_96 = self.builder.get_object("label96")
        self.label_96.set_name("label96")
        self.label_96.set_text(self._("When the computer is online, NFS Desktop OS can automatically check for critical updates. You can choose to automatically install these updates or just send notifycation to you."))
        self.label_98 = self.builder.get_object("label98")
        self.label_98.set_name("label98")
        self.label_98.set_text(self._("Install the new update:"))
        self.label_99 = self.builder.get_object("label99")
        self.label_99.set_name("label99")
        self.label_99.set_text(self._("at"))
        self.label_3 = self.builder.get_object("label3")
        self.label_3.set_name("label3")
        self.label_3.set_text(self._("current system version:"))
        self.label_4 = self.builder.get_object("label4")
        self.label_4.set_name("label4")
        self.label_4.set_text(self._("lastest system version:"))
        self.label_5 = self.builder.get_object("label5")
        self.label_5.set_name("label5")
        self.label_5.set_text(self._("receive upgrade:"))
        with open ("/etc/os-release") as f:
            for line in f.readlines():
                if line.startswith("VERSION="):
                    cur_release = eval(line.split("=")[-1].strip("\n"))        
        self.label_6 = self.builder.get_object("label6")
        self.label_6.set_name("label6")
        self.label_6.set_text(cur_release)
        self.label_7 = self.builder.get_object("label7")
        self.label_7.set_name("label7")
        self.label_7.set_text("")
        self.label_8 = self.builder.get_object("label8")
        self.label_8.set_name("label8")
        self.label_8.set_text(self._("Only applies to NFS Desktop OS"))
        
        self.label_upgrade = self.builder.get_object("upgrade_title")
        self.label_upgrade.set_name("upgrade_title")
        self.label_upgrade.set_text(self._("NFS Desktop OS Upgrade"))
        self.status_image1 = self.builder.get_object("status_image1")
        self.status_image1.set_name("status_image1")
        self.status_image1.set_visible(False)
        self.label_tip = self.builder.get_object("upgrade_tip")
        self.label_tip.set_name("upgrade_tip")
        self.label_tip.set_text(self._("check whether if updated version system can be upgraded"))        
        
        self.upgrade_btn = self.builder.get_object("enterbutton1")
        self.upgrade_btn.set_name("enterbutton1")
        self.upgrade_btn.set_label(self._("Check Update"))
        self.upgrade_btn.connect("clicked", self.on_upgrade_click)
        #self.lable3 = self.builder.get_object("label3")
        #self.label3.set_name("label3")
        #self.label3.set_text(self._("Through the network to upgrade the system, please ensure that you have a connection network"))
        
        self.eventbox18 = self.builder.get_object('eventbox18')
        self.eventbox18.set_name('eventbox18')
        self.confirm_btn = self.builder.get_object('confirm')
        self.confirm_btn.set_name('confirm')
        self.confirm_btn.set_label(self._('confirm'))
        self.confirm_btn.connect("button_press_event", self.on_confirm_press)
        self.confirm_btn.connect("clicked", self.on_confirm_click)
        self.cancel_btn = self.builder.get_object('cancel')
        self.cancel_btn.set_name('cancel')
        self.cancel_btn.set_label(self._('cancel'))
        self.cancel_btn.connect("clicked", self.on_cancel_click)
        
        screen  = Gdk.Screen.get_default()
        self.window.set_size_request(200, 150)
        self.window.set_position(Gtk.WindowPosition.CENTER)
        self.window.connect("delete-event", self.hide)
        
    def show(self):
        self.window.present()
        self.window.show_all()

    def hide(self, *args):
        if self.confirm_btn.get_sensitive() == True:
            self.on_cancel_click(None)
        self.window.hide()
        return True

    def showInitPage(self, checkCallback):
        self.status_text.set_markup(self._("Check whether there are new updates for NFS Desktop OS."))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Check Update"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            self.showChecking()
            checkCallback(True)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(False)
        self.status_detail_text.set_text("")
        self.confirm_btn.set_sensitive(False)
        self.cancel_btn.set_sensitive(False)

    def showUpdated(self, checkCallback):
        self.status_text.set_markup(self._("Install Updates"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Check Update"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            self.showChecking()
            checkCallback(True)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(self._("Latest updates currently installed"))

    def showChecking(self):
        self.status_text.set_markup(self._("Checking updates......"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(True)
        self.progress_update.set_fraction(0.5)
        self.enterbutton.set_visible(False)
        self.status_detail_text.set_visible(False)

    def showCheckError(self, checkCallback):
        self.status_text.set_markup(self._("Can not search updates"))
        self.status_text_image.set_from_file("/usr/share/cdos-update/frontend/img/err.png")
        self.status_text_image.set_visible(True)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Check Update"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            self.showChecking()
            checkCallback(True)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(False)

    def showToInstall(self, toDoList, installCallback):
        self.status_text.set_markup(self._("Download and install updates of the computer"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Install Updates"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            installCallback(toDoList)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(self._("Total") + " " + str(len(toDoList)) + " " + self._("updates need to download and install"))

    def showToInstallAfterDownload(self, toDoList, installCallback):
        self.status_text.set_markup(self._("install the downloaded updates"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Install Updates"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            installCallback(toDoList)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(self._("Total") + " " + str(len(toDoList)) + " " + self._("updates can be installed"))

    def showDownloading(self, status_text, progress):
        self.status_text.set_markup(self._("downloading updates"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(True)
        self.progress_update.set_fraction(progress / 100.0)
        self.enterbutton.set_visible(False)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(status_text)

    def showUpdating(self, status_text, progress):
        self.status_text.set_markup(self._("Installing updates"))
        self.status_text_image.set_visible(False)
        self.progress_update.set_visible(True)
        self.progress_update.set_fraction(progress / 100.0)
        self.enterbutton.set_visible(False)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(status_text)

    def showAllInstalled(self, un_install_patch_list, checkCallback):
        self.status_text.set_markup(self._("Install updates successfully"))
        self.status_text_image.set_from_file("/usr/share/cdos-update/frontend/img/correct.png")
        self.status_text_image.set_visible(True)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Check Update"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            self.showChecking()
            checkCallback(True)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        if len(un_install_patch_list) == 0:
            self.status_detail_text.set_visible(False)
        if len(un_install_patch_list) > 0:
            self.status_detail_text.set_visible(True)
            if len(un_install_patch_list) == 1:
                status_text_string = (self._("There is %s update cannot install because of the following reasons:")%str(len(un_install_patch_list)) + "\n")
            else:
                status_text_string = (self._("There are %s updates cannot install ecause of the following reasons:")%str(len(un_install_patch_list)) + "\n")
            status_text_string += (self._("1. some packages are missing which updates is depends on") + "\n")
            status_text_string += (self._("2. download untrusted updates") + "\n")
            status_text_string += (self._("3. updates installed with errors") + "\n")
            self.status_detail_text.set_markup(status_text_string)

    def showError(self, status_text, checkCallback):
        self.status_text.set_markup(self._("update failed"))
        self.status_text_image.set_from_file("/usr/share/cdos-update/frontend/img/err.png")
        self.status_text_image.set_visible(True)
        self.progress_update.set_visible(False)
        self.enterbutton.set_label(self._("Check Update"))
        self.enterbutton.set_visible(True)
        def enterCallback(*argv):
            self.showChecking()
            checkCallback(True)
        if self.lastEnterButtonSignal != None:
            self.enterbutton.disconnect(self.lastEnterButtonSignal)
        self.lastEnterButtonSignal = self.enterbutton.connect("clicked", enterCallback)
        self.status_detail_text.set_visible(True)
        self.status_detail_text.set_markup(status_text)

    def setFrequecy(self, willcheckupdate, autodownload, autoinstall):
        frequecy = None
        if not willcheckupdate:
            frequecy = self.updateFrequenceArray[3]
        else:
            if autoinstall:
                frequecy = self.updateFrequenceArray[0]
            elif autodownload:
                frequecy = self.updateFrequenceArray[1]
            else:
                frequecy = self.updateFrequenceArray[2]

        iterator = self.updateFrequence_list_store.get_iter_first()
        while iterator:
            if self.updateFrequence_list_store[iterator][0] == frequecy:
                if iterator != self.updatefrequence.get_active_iter():
                    self.updatefrequence.set_active_iter(iterator)
                break
            iterator = self.updateFrequence_list_store.iter_next(iterator)
        self.set_updatefrequence()

    # def on_changed_updatefrequence(self, combobox):
    def set_updatefrequence(self):
        combobox = self.updatefrequence
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.updateFrequenceArray)):
            if self.updateFrequenceArray[i] == model[iterator][0]:
                chosen_item = i
                break
        willcheckupdate = True
        autodownload = False
        autoinstall = False
        if chosen_item >= 0:
            if chosen_item == 0:
                willcheckupdate = True
                autodownload = True
                autoinstall = True
                if not self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(True)
                    self.hourfreq.set_sensitive(True)
            elif chosen_item == 1:
                willcheckupdate = True
                autodownload = True
                autoinstall = False
                if not self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(True)
                    self.hourfreq.set_sensitive(True)
            elif chosen_item == 2:
                willcheckupdate = True
                autodownload = False
                autoinstall = False
                if not self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(True)
                    self.hourfreq.set_sensitive(True)
            elif chosen_item == 3:
                willcheckupdate = False
                autodownload = False
                autoinstall = False
                if self.dayfreq.is_sensitive():
                    self.dayfreq.set_sensitive(False)
                    self.hourfreq.set_sensitive(False)
            else:
                return
        else:
            return
        schema = Gio.Settings(schema = "org.cdos.update")
        schema.set_boolean("autoinstall", autoinstall)
        schema.set_boolean("autodownload", autodownload)
        schema.set_boolean("checkupdate", willcheckupdate)

    def  on_checkupdate_clicked(self, combobox):
        self.checkupdate.get_style_context().add_class('tab_active_button')
        self.changesettings.get_style_context().remove_class('tab_active_button')
        self.checkupgrade.get_style_context().remove_class('tab_active_button')
        self.notepage.set_current_page(self.progress_pages['checkupdate'])
        
    def  on_checkupgrade_clicked(self, combobox):
        self.checkupgrade.get_style_context().add_class('tab_active_button')
        self.checkupdate.get_style_context().remove_class('tab_active_button')
        self.changesettings.get_style_context().remove_class('tab_active_button')
        self.notepage.set_current_page(self.progress_pages['checkupgrade'])
       
    def  on_changesettings_clicked(self, combobox):
        self.changesettings.get_style_context().add_class('tab_active_button')
        self.checkupdate.get_style_context().remove_class('tab_active_button')
        self.checkupgrade.get_style_context().remove_class('tab_active_button')
        self.notepage.set_current_page(self.progress_pages['settings'])

    def setDayFrequecy(self, dayfreq, hourfreq):
        iterator = self.dayfreq_list_store.get_iter_first()
        while iterator:
            if self.dayfreq_list_store[iterator][0] == self.dayfreqArray[dayfreq]:
                if iterator != self.dayfreq.get_active_iter():
                    self.dayfreq.set_active_iter(iterator)
                break
            iterator = self.dayfreq_list_store.iter_next(iterator)

        iterator = self.hourfreq_list_store.get_iter_first()
        while iterator:
            if self.hourfreq_list_store[iterator][0] == (str(hourfreq)+":00"):
                if iterator != self.hourfreq.get_active_iter():
                    self.hourfreq.set_active_iter(iterator)
                break
            iterator = self.hourfreq_list_store.iter_next(iterator)

    # def on_changed_dayfreq(self, combobox):
    def set_dayfreq(self):
        combobox = self.dayfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.dayfreqArray)):
            if self.dayfreqArray[i] == model[iterator][0]:
                chosen_item = i
                break
        if chosen_item >= 0:
            schema = Gio.Settings(schema = "org.cdos.update")
            schema.set_int("check-day", chosen_item)

    # def on_changed_hourfreq(self, combobox):
    def set_hourfreq(self):
        combobox = self.hourfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        schema = Gio.Settings(schema = "org.cdos.update")
        schema.set_int("check-hour", int(model[iterator][0].split(':')[0]))

    def on_changed_updatefrequence(self, combobox):
        if (self.dayfreq.get_sensitive() == False) or (self.hourfreq.get_sensitive() == False):
            self.dayfreq.set_sensitive(True)
            self.hourfreq.set_sensitive(True)
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_dayfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_hourfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def set_last_checkTime(self,check_last_time):
        if check_last_time != 0:
            check_last_time  = time.strftime( ISOTIMEFORMAT, time.localtime(float(check_last_time)))
        else:
            check_last_time = ''
        self.current_check_time_label.set_text(check_last_time)
   
    def set_last_installTime(self,install_last_time):
        if install_last_time != 0:
            install_last_time = time.strftime( ISOTIMEFORMAT, time.localtime(float(install_last_time)))
        else:
            install_last_time = ''
        self.last_install_time_label.set_text(install_last_time)
        
    def on_confirm_press(self, *args):
        self.set_updatefrequence()
        self.set_dayfreq()
        self.set_hourfreq()

    def on_confirm_click(self, button):
        self.confirm_btn.set_sensitive(False)
        self.cancel_btn.set_sensitive(False)

    def on_cancel_click(self, button):
        schema = Gio.Settings(schema = "org.cdos.update")
        dayfreq = schema.get_int("check-day")
        hourfreq = schema.get_int("check-hour")
        self.setDayFrequecy(dayfreq, hourfreq)
        autoinstall = schema.get_boolean("autoinstall")
        autodownload = schema.get_boolean("autodownload")
        willcheckupdate = schema.get_boolean("checkupdate")
        self.setFrequecy(willcheckupdate, autodownload, autoinstall)
        self.confirm_btn.set_sensitive(False)
        self.cancel_btn.set_sensitive(False)
        # self.hide()

    def on_upgrade_click(self, button):
        rc = check_need_upgrade()
        if rc == 1:
            cur_release, new_release = get_current_and_new_version()
            self.label_7.set_text(new_release)
            if not get_upgrade_configfile(new_release):
                return
            self.status_image1.set_visible(False)
            self.label_tip.set_markup(self._("check whether if updated version system can be upgraded"))
            dialog = UpgradeDialog(new_release)
            dialog.show()
        elif rc == 0:
            cur_release, new_release = get_current_and_new_version()
            self.label_7.set_text(cur_release)
            self.status_image1.set_visible(False)
            self.label_tip.set_markup(self._("the current version is the latest version"))
        elif rc == 2:
            cur_release, new_release = get_current_and_new_version()
            self.label_7.set_text(new_release)
            self.status_image1.set_visible(False)
            self.label_tip.set_markup(self._("the current version can not upgrade"))
        else:
            self.label_tip.set_markup(self._("Can not search updates"))
            self.status_image1.set_from_file("/usr/share/cdos-update/frontend/img/err.png")
            self.status_image1.set_visible(True)

