#! /bin/python3
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sys

import time
import os
import logging
import tempfile
import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from gi.repository.GdkPixbuf import Pixbuf,PixbufAnimation
from check_upgrade import *
from warning import *
import _thread
from history import history_db

from dbus_updates import License_dbusInterface

import json
import shutil
from patch_lib import getLastCheckTime
from patch_lib import setLastCheckTime
import urllib.request
import gettext

gettext.bindtextdomain('cdosupdate', "/usr/share/locale/")
gettext.textdomain('cdosupdate')
_ = gettext.gettext

ISOTIMEFORMAT = '%Y-%m-%d %H:%M'

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename=tempfile.mktemp("cdos-update-frontend.log"),level=logging.DEBUG)

class Result:
    __error = None
    __value = None

    def __init__(self, error, value):
        self.__error = error
        self.__value = value

    def getError(self):
        return self.__error

    def getValue(self):
        return self.__value

class PatchStatus:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

class CheckMainwindow(object):
    
    def __init__(self, img_dir ,uidir):

        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\

            #note-viewport {
                background-color: @theme_bg_color;
            }

            #main_notebook {
                border: none;
            }
            #setting_button,
            #import_button,
            #detail_button {
                border: none;
                color: #007dce;
                font-size: 12px;
                background: transparent;
            }
            #detail_button:active{
                border: none;
            }
            #update_name {
                font-size: 18px;
                color: #333;
            }
            #update_name_msg,
            #detail_label {
                font-size: 12px;
                color: #888;
            }
            #update_button {
                font-size: 14px;
                color: #333;
            }

            #update_button:insensitive {
                background: #cccccc;
                color: #FFFFFF;
                border: none;
            }

            #update_button,
            #loadingfailed_button,
            #enterbutton,
            #enterbutton1,
            #confirm {
                -GtkWidget-focus-padding: 0;
                -GtkWidget-focus-line-width: 0;
                -GtkButton-inner-border: 0;

                background: #44A6E5;
                color: #FFFFFF;
                box-shadow: none;
                font-weight: normal;
                font-size: 13px;
                border: none;
                border-radius: 0;
                border-color: #44A6E4;
                border-width: 1px;
                border-style: solid;
            }

            #update_button:active,
            #update_button:active:hover,
            #loadingfailed_button:active,
            #loadingfailed_button:active:hover,
            #enterbutton:active,
            #enterbutton:active:hover,
            #enterbutton1:active,
            #enterbutton1:active:hover,
            #confirm:active,
            #confirm:active:hover {
                background: #3C93D3;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #update_button:hover,
            #loadingfailed_button:hover,
            #enterbutton:hover,
            #enterbutton1:hover,
            #confirm:hover {
                background: #48B2ED;
                color: #FFFFFF;
                box-shadow: 0px 0px 0px 0px;
            }

            #update_button:focus,
            #update_button:hover:focus,
            #update_button:active:focus,
            #loadingfailed_button:focus,
            #loadingfailed_button:hover:focus,
            #loadingfailed_button:active:focus,
            #enterbutton:focus,
            #enterbutton:hover:focus,
            #enterbutton:active:focus,
            #enterbutton1:focus,
            #enterbutton1:hover:focus,
            #enterbutton1:active:focus,
            #confirm:focus,
            #confirm:hover:focus,
            #confirm:active:focus,
            #cancel:focus,
            #cancel:hover:focus,
            #cancel:active:focus {
                border-color: shade(#44A6E4, 0.8);
                box-shadow: 0px 0px 0px 0px;
            }

            #confirm:insensitive,
            #confirm :insensitive,
            #cancel:insensitive,
            #cancel :insensitive {
                background: #cccccc;
                color: #FFFFFF;
                border: none;
            }
            #backspace_object {
                background: #f4f1f4;
                padding: 15px 15px 15px 15px;
            }
            #last_update_button {
                color: #007dce;
                background: @theme_bg_color;
                font-size: 14px;
                box-shadow: none;
                border: none;
                margin-right: 655px;
                padding-top: 15px;
                padding-left: 0px;
                padding-bottom: 10px;
            }
            #loadingfailed_label {
                color: #888888;
            }
            #loadingfailed_detail_label {
                color: #999999;
            }

            #check_update_button {
                background: #d1d1d1;
                color: black;
                font-size: 13px;
                border: none;
            }
            #check_update_button:hover {
                background: #dddddd;
            }
            #check_update_button:active {
                background: #cccccc;
            }

            #waiting_install_button {
                background: #d1d1d1;
                color: white;
                font-size: 13px;
                border: none;
            }

            #ok_button {
                background: #44A6E5;
                color: white;
                font-size: 14px;
                border: none;
            }
            #ok_button:hover {
                background: #48B2ED;
                color: #FFFFFF;
            }
           
            #ok_button:active {
                background: #34A6E4;
            }

            #cancel_button {
                background: #bbbbbb;
                color: black;
                font-size: 14px;
                border: none;
            }
            #cancel_button:hover {
                background: #cccccc;
            }
            
            #cancel_button:active {
                background: #afafaf;
            }

            #label_desc {
                color: #888;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.join(uidir, "mainwindow3.ui"))
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("mainwindow")
        self.window.set_title(_("NFS Desktop OS Update"))
        self.window.set_size_request(960, 540)

        # window_icon = "/usr/share/cdos-update/frontend/ui/logo.png"
        # if os.path.exists(window_icon):
        #     pixbuf = Pixbuf.new_from_file_at_size(window_icon, 54, 54)
        #     self.window.set_icon(pixbuf)
        pixbuf = self.window.get_icon()
        pixbuf = pixbuf.scale_simple(54, 54, 2)
        self.window.set_icon(pixbuf)

        self.note_viewport = self.builder.get_object("note-viewport")
        self.note_viewport.set_name("note-viewport")
        self.notepage = self.builder.get_object("notebook_page")
        self.notepage.set_name("notebook_page")
        self.progress_pages = {
            'main': 0,
            'settings': 1,
        }

        self.img_dir = img_dir
        self.uidir = uidir
        self.app_num = 0
        # update info
        self.updates_handler_json = {
            "upgrade": None,
            "system": None,
            "apps": {}
        }
        self.app_query = []

        self.current_work_type = ""
        self.current_work = {}
        self.busy_flag = False
        self.upgrade_flag = False
        self.env = {}
        self.env["HOME"] = os.getenv("HOME")
        self.env["USER"] = os.getenv("USER")
        self.env["DISPLAY"] = os.getenv("DISPLAY")
        self.env["PATH"] = os.getenv("PATH")
        # query need to be exec
        self.query = []
        self.current_obj = None

        self.import_info = {}
        self.update_tips = None
        self.updates_check_obj = None
        self.update_items = []
        self.working_items = []
        self.schema = Gio.Settings(schema = "org.cdos.update")
        
        try:
            self.history_db = history_db()
        except Exception as e:
            logging.error("history db:" + str(e))
            self.history_db = None
        # init url for updates
        self.init_updates_url()
        self._init_dbus()
        self._init_page()
        self._init_daemon_handler()
        #self.init_notebook_pages()

        self.window.connect("delete-event", self.quit_window)

    def quit_window(self, widget, event):
        if not len(self.working_items):
            self.do_close()
        else:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(_("NFS Desktop OS Update is working.\nClose it later."))
            warning_dialog.show()
            return True

    def do_close(self):
        Gtk.main_quit()

    def init_updates_url(self):
        #self.repo_url = "http://repos.01os.org/"
        #self.site_path = self.repo_url + "data/cdos-update/update/"
        # self.repo_url = "http://172.30.13.37/"
        # self.site_path = self.repo_url + "patches/"
        # self.getpatch_site_path = self.site_path + "getpatch/"
        # self.getpatchlist_site_path = self.site_path + "getpatchlist"
        # self.getpatchlistinfo_site_patch = self.site_path + "getpatchesinfoWithSystemVersion"
        # self.upgrade_path = self.repo_url + "data/cdos-update/cdos-upgrade-new-server/meta-release"

        self.repo_url = "https://update.os.nfschina.com/static/"
        self.new_repo_url = "https://repos.os.nfschina.com"
        self.patch_url = self.new_repo_url + "/patch-server/patch_update.json"

        self.site_path = self.repo_url + ":2367/"
        self.getpatch_site_path = self.site_path + "getpatch/"
        self.getpatchlist_site_path = self.site_path + "getpatchlist"
        self.getpatchlistinfo_site_patch = self.site_path + "getpatchesinfoWithSystemVersion"
        self.upgrade_path = self.repo_url + ":8080/data/upgrade/cdos-upgrade/"

    def _init_dbus(self):
        intern_dbus_name = "org.cdos.update" 
        intern_dbus_path = "/org/cdos/update"
        intern_dbus_iface = "org.cdos.update"
        self.system_bus = dbus.SystemBus()
        self.remote_obj = self.system_bus.get_object( intern_dbus_name, intern_dbus_path )
        self.iface = dbus.Interface( self.remote_obj, intern_dbus_iface)
        self.is_rollback = False

    def _init_daemon_handler(self):
        self.system_bus.add_signal_receiver(self.handle_statusChanged,'statusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_aptStatusChanged,'AptStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_upgradeStatusChanged,'upgradeStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_cachestatusChanged,'cacheStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_patchStatusChanged,'patchStatusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        self.system_bus.add_signal_receiver(self.handle_backup_recover_status_changed,'backup_recover_status_changed', "org.cdos.update", "org.cdos.update", "/org/cdos/update")

    def _init_page(self):
        self.main_notebook = self.builder.get_object("main_notebook")
        self.main_notebook.set_name("main_notebook")
        self.main_notebook_pages = {
            'loading': 0,
            'loadingfailed': 1,
            'loadingsucceed': 2,
        }
        self.main_notebook.set_current_page(self.main_notebook_pages['loadingsucceed'])

        self.init_setting_page()

        self.top_label = self.builder.get_object("top_label")
        self.top_label.set_name("top_label")
        self.top_label.set_text(_("System Update"))
        self.top_label.show()

        self.setting_button = self.builder.get_object("setting_button")
        self.setting_button.set_name("setting_button")
        self.setting_button.set_label(_("settings..."))
        #self.loading_succeed_page(info=info)

        self.import_button = self.builder.get_object("import_button")
        self.import_button.set_name("import_button")
        self.import_button.set_label(_("Import Update Package"))
        
        self.notepage.set_current_page(self.progress_pages['main'])

        self.loadingsucceed_box = self.builder.get_object("loadingsucceed_box")
        self.loadingsucceed_box.set_name("loadingsucceed_box")
        self.loadingsucceed_box.set_halign(Gtk.Align.FILL)
        self.loadingsucceed_box.set_valign(Gtk.Align.START)
        self.loadingsucceed_grid = Gtk.Grid()

        separator = Gtk.HSeparator()
        separator.show()
        self.loadingsucceed_box.pack_end(self.loadingsucceed_grid, False, False, 0)
        self.loadingsucceed_grid.attach(separator, 0, 0, 4, 1)

        image_name = os.path.join( self.img_dir, "cdos.png")
        def upgrade_event_callback():
            if len(self.working_items):
                TipsDialog(_("Tips"), self.window,
                                _("An update item is being updated. Please wait..."), 
                                "",
                                callback_cancel=False)
                self.updates_check_obj.check_update_end()
                return
            if not self.check_network_connection():
                if not self.check_network_connection():
                    TipsDialog(_("Tips"), self.window,
                                    _("Unable to connect to the server"), 
                                    _("Please check the network!"),
                                    callback_cancel=False)
                    self.updates_check_obj.check_update_end()
                    return

            for item in reversed(self.update_items):
                self.loadingsucceed_grid.remove_row(item.row)
                self.update_items.remove(item)

            self.update_cache()
            #GLib.idle_add(self.update_cache)
            #self.get_update_data()

        self.updates_check_obj = Updates_Check_Object(image_name, button_callback=upgrade_event_callback)
        self.updates_handler_json["upgrade"] = self.updates_check_obj
        self.loadingsucceed_grid.attach(self.updates_check_obj, 0, 1, 4, 1)
        self.loadingsucceed_grid.show()
        self.show()

        self.updates_check_obj.check_update_start(None)

    def update_cache(self):
        self.iface.update_cache(self.env)

    def handle_cachestatusChanged(self, msg):
        #GLib.idle_add(self.get_update_data)
        self.get_update_data()

    def commit_system_update(self, getpatch_info_site_path, getpatch_site_path, pkgs_install=[], pkgs_upgrade=[]):
        self.iface.commit_system_updates( self.env, pkgs_install, pkgs_upgrade, getpatch_info_site_path, getpatch_site_path)

    def commit_app_update(self, pkgs_install=[], pkgs_upgrade=[]):
        self.iface.commit_app_pkgs( self.env, pkgs_install, pkgs_upgrade)

    #没有被调用
    def install_patches(self, getpatchlistinfo_site_patch, getpatch_site_path):
        self.iface.install( getpatch_site_path, self.iface.get_patchlist_to_install( getpatchlistinfo_site_patch))
        return

    #没有被调用
    def get_patches_to_install(self, getpatchlistinfo_site_patch):
        return self.iface.get_patchlist_to_install( getpatchlistinfo_site_patch)

    def exec_query(self):
        if self.busy_flag:
            return
        else:
            self.busy_flag = True
            self.do_exec_query()

    def do_exec_query(self):
        if len(self.query) > 0:
            work = self.query.pop(0)
            if work["type"] == "system_update":
                self.current_work_type = "system_update"
                self.current_work = work["ev_info"]
                self.current_obj = self.updates_handler_json["system"]
                self.commit_system_update( getpatch_info_site_path=self.getpatchlistinfo_site_patch, getpatch_site_path=self.getpatch_site_path, pkgs_install=list(work["ev_info"]["debs"]))
            elif work["type"] == "app_update":
                self.current_work_type = "app_update"
                self.current_work = work["ev_info"]
                self.current_obj = self.updates_handler_json["apps"][work["ev_info"]["debs"][0]]
                # install app deb
                self.commit_app_update( pkgs_install=list(work["ev_info"]["debs"]))

    def push_query(self, ev):
        self.query.append(ev)
        self.exec_query()

    def pop_query(self, ev):
        if ev in self.query:
            self.query.remove(ev)
            if ev["cancel_obj"]:
                ev["cancel_obj"].cancel_func()
        else:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(_("NFS Desktop OS Update is working.\nTry it later."))
            warning_dialog.show()

    #接收到daemon的upgradeStatusChanged信号后执行的函数.
    def handle_upgradeStatusChanged(self, statusCode, status_text, paramList):
        def dist_upgrade(upgrade_cmd):
            logging.info( upgrade_cmd)
            os.system("bash " + upgrade_cmd + " show")

        self.updates_handler_json["upgrade"].show_progress_box()
        self.updates_handler_json["upgrade"].progressbar_info_label.set_text(_(status_text))
        if statusCode == 101:
            self.upgrade_flag = False
            self.set_update_sensitive(True)
            logging.info( "statusCode is 101")
            self.updates_handler_json["upgrade"].show_button()
            # ToDo: restore status
            return
        self.updates_handler_json["upgrade"].progressbar.set_fraction(statusCode/100.0)
        if statusCode == 100:
            try:
                upgrade_tool_main_path = os.path.join(paramList["upgrade_tool_dir"], "distupgrade")
                #调用升级工具中的distupgrade脚本.
                thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
                time.sleep(1)
            except Exception as e:
                logging.error("handle_upgradeStatusChanged:" + str(e))
            finally:
                self.do_close()
                pass

    #补丁安装信号变化.
    def handle_statusChanged(self, statusCode, status_text, patchID, correct_install_patches_list, patchList, progress, paramList):
        current_obj = self.updates_handler_json["system"]
        if statusCode == PatchStatus.ERROR_INSTALL:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir, True)
            warning_dialog.set_label(_("Error occurs when patches installed.\nClose the window."))
            warning_dialog.show()
            return
        if statusCode == PatchStatus.ALLINSTALLED:
            current_obj.set_progress_bar(1.0)
            current_obj.finish()
            # history system
            update_length = len(current_obj.deb_list)+len(correct_install_patches_list)
            if update_length == 1:
                short_desc = _("Install %d update") %update_length
            else:
                short_desc = _("Install %d updates") %update_length
            long_desc = ""
            if not current_obj.history_long_desc == "":
                long_desc = current_obj.history_long_desc + "\n"
            if len(correct_install_patches_list) == 0:
                pass
            elif len(correct_install_patches_list) == 1:
                long_desc += _("Install %d patch:") %len(correct_install_patches_list)
            else:
                long_desc += _("Install %d patches:") %len(correct_install_patches_list)
            for patch in correct_install_patches_list:
                long_desc += " " + str(patch)
            try:
                self.history_db.insert(update_type="systemupdate", updateTime=int(time.time()), appname=current_obj.update_name, version="", size=current_obj.size, icon=current_obj.image_name, shortdesc=short_desc, longdesc=long_desc)
                self.inster_history(update_type="update", image_name=current_obj.image_name, update_name=current_obj.update_name, update_name_msg=short_desc, show_detail=True, detail_info=long_desc)
            except Exception as e:
                logging.info("insert history Exception")
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
        else:
            current_obj.set_progress_bar(80.0 + float(progress)*0.2)

    #apt状态变化，主要是系统组件更新和应用更新.
    def handle_aptStatusChanged(self, status):
        if status["type"] == "app_update":
            if status["first_deb"] in self.updates_handler_json["apps"]:
                current_obj = self.updates_handler_json["apps"][status["first_deb"]]
        elif status["type"] == "system_update":
            current_obj = self.updates_handler_json["system"]
        else:
            return
        current_obj.hide_button()

        if status["status"] == "inprogress":
            if status["type"] == "system_update":
                current_obj.set_progress_bar(status["percent"]*0.8)
            else:
                current_obj.set_progress_bar(status["percent"])
        elif status["status"] == "success" and status["type"] == "app_update":
            current_obj.finish()

            self.app_num = self.app_num - 1
            if self.app_num <= 0:
                self.app_title_grid.hide()
            elif self.app_num == 1:
                self.app_label.set_text(_("%d application to be updated") %self.app_num)
            else:
                self.app_label.set_text(_("%d applications to be updated") %self.app_num)
            # history app
            try:
                self.history_db.insert(update_type="application", updateTime=int(time.time()), appname=current_obj.update_name, version=current_obj.version, size=current_obj.size, icon=current_obj.image_name, shortdesc=current_obj.update_name_msg, longdesc=current_obj.detail_info)
                self.inster_history(update_type="app", image_name=current_obj.image_name, update_name=current_obj.update_name, update_name_msg=current_obj.update_name_msg, show_detail=True, detail_info=current_obj.history_long_desc)
            except Exception as e:
                logging.info("insert history Exception")
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
        elif status["status"] == "failed":
            if status['error'] == 'download failed':
                self.on_download_failed( current_obj)
                return
            current_obj.finish_exec()
            #error_msg = _("Error occurs when debs installed.\nRun \"sudo apt-get install -f\" to fix it.")
            error_msg = status['error']
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(error_msg)
            warning_dialog.show()
            if len(self.query) > 0:
                self.do_exec_query()
            else:
                self.busy_flag = False
            return

    #deb包安装的下载失败的处理，只针对下载失败的错误.
    def on_download_failed(self, current_obj):
        current_obj.finish_exec()
        self.current_work_type = ""
        self.busy_flag = False
        while len(self.query) > 0:
            work = self.query.pop(0)
            one_obj = None
            if work["type"] == "system_update":
                one_obj = self.updates_handler_json["system"]
            elif work["type"] == "app_update":
                one_obj = self.updates_handler_json["apps"][work["ev_info"]["debs"][0]]
            if one_obj is not None:
                one_obj.finish_exec()
        error_msg = _('Download failed, please check the network connection.')
        warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
        warning_dialog.set_label(error_msg)
        warning_dialog.show()

    def get_dbus_daemon(self):
        return self.iface

    #调用daemon的getAllUpdates的dbus接口,获取到补丁、deb包和系统升级信息的json
    def get_updates_info(self):
        try:
            res = {}
            res = self.iface.getAllUpdates( os.environ["HOME"] ,self.upgrade_path, self.getpatchlistinfo_site_patch)
            return Result(None, res)
        except Exception as e:
            logging.info("get_updates_info Exception:" + str(e))
            return Result("get_updates_info error:" + str(e), None)

    def show(self):
        self.window.show()

    # main page
    def go_to_main_page(self):
        self.notepage.set_current_page(self.progress_pages['main'])

    def check_network(self):
        # 无法检测404异常，仅用来检测是否有网络链接
        res = None
        try:
            res = urllib.request.urlopen(self.repo_url)
        except Exception as e:
            logging.error(str(e))
            return False
        content = str(res.read())
        if content == 'fail':
            return False
        return True

    # main page -> sucess page
    def go_to_updates_page(self):
        #加载更新信息
        def load_updates_info(self):
            if not self.check_network():
                time.sleep(1)
                self.go_to_loadingfailed_page()
                return
            updates_info = self.get_updates_info()
            # error or no value
            if (updates_info.getError() is not None) or (updates_info.getValue() is None):
                self.go_to_loadingfailed_page()
                return

            updates_info_value = updates_info.getValue()
            def dist_upgrade(upgrade_cmd):
                logging.info("upgrade_cmd:" + upgrade_cmd)
                os.system("bash " + upgrade_cmd + " show")
            try:
                # if is downloading/upgrading
                if ( "_upgrade" in updates_info_value) and ("upgrading" in updates_info_value["_upgrade"]) and (updates_info_value["_upgrade"]["upgrading"]) and ( "upgrade_tool_dir" in updates_info_value["_upgrade"]) and ( not updates_info_value["_upgrade"]["upgrade_tool_dir"] == ""):
                    try:
                        upgrade_tool_main_path = os.path.join( updates_info_value["_upgrade"]["upgrade_tool_dir"], "distupgrade")
                        thread.start_new_thread(dist_upgrade, (upgrade_tool_main_path, ))
                        time.sleep(1)
                    except Exception as e:
                        logging.error("load_updates_info2:" + str(e))
                    finally:
                        self.do_close()
            except Exception as e:
                pass

            GLib.idle_add(self.go_to_loadingsucceed_page, updates_info.getValue())
        thread.start_new_thread(load_updates_info, (self,))

    # setting page
    def go_to_settings_page(self):
        self.notepage.set_current_page(self.progress_pages['settings'])

    def init_notebook_pages(self):
        self.init_main_page()
        self.init_setting_page()

    #主要页面，显示系统升级和系统更新信息的页面.
    def init_main_page(self):
        self.main_notebook = self.builder.get_object("main_notebook")
        self.main_notebook.set_name("main_notebook")
        self.main_notebook_pages = {
            'loading': 0,
            'loadingfailed': 1,
            'loadingsucceed': 2,
        }

        #左侧说明文字和设置按钮.
        self.setting_button = self.builder.get_object("setting_button")
        self.setting_button.set_name("setting_button")
        self.setting_button.set_label(_("settings..."))
        self.top_label = self.builder.get_object("top_label")
        self.top_label.set_name("top_label")

        #正在检查更新文字.
        self.loading_label = self.builder.get_object("loading_label")
        self.loading_label.set_name("loading_label")
        self.loading_label.set_text(_("Checking updates..."))

        #检查更新的gif图.
        self.loading_image = self.builder.get_object("loading_image")
        self.loading_image.set_name("loading_image")
        loading_image_wigth = 20
        loading_image_height = 20
        # loading_image_name = "/usr/share/cdos-update/frontend/img/loading.gif"
        loading_image_name = os.path.join( self.img_dir, "loading.gif")
        loading_pixbuf = PixbufAnimation.new_from_file( loading_image_name)
        self.loading_image.set_from_animation(loading_pixbuf)

        #检查失败的页面.
        self.loadingfailed_label = self.builder.get_object("loadingfailed_label")
        self.loadingfailed_label.set_name("loadingfailed_label")
        self.loadingfailed_label.set_text(_("Check updates failed!"))
        self.loadingfailed_button = self.builder.get_object("loadingfailed_button")
        self.loadingfailed_button.set_name("loadingfailed_button")
        self.loadingfailed_button.set_label(_("Check again"))
        self.loadingfailed_button.connect('clicked', self.on_reloading)#重新检查按钮

        self.loadingfailed_detail_label = self.builder.get_object("loadingfailed_detail_label")
        self.loadingfailed_detail_label.set_name("loadingfailed_detail_label")
        failed_datail_info = _("Possible reasons:")
        failed_datail_info += "\n" + "1." + _("Network connection exception, pelase check it.")
        failed_datail_info += "\n" + "2." + _("Exception for deb dependency, you can run \"sudo apt-get install -f\" to fix it.")
        failed_datail_info += "\n" + "3." + _("Unknown exceptions, try it again.")
        self.loadingfailed_detail_label.set_text(failed_datail_info)

    def on_reloading(self, widget):
        self.go_to_loading_page()
        self.go_to_updates_page()

    # loading
    def go_to_loading_page(self):
        self.main_notebook.set_current_page(self.main_notebook_pages['loading'])
        self.notepage.set_current_page(self.progress_pages['main'])
        self.top_label.hide()

    # loading failed
    def go_to_loadingfailed_page(self):
        self.main_notebook.set_current_page(self.main_notebook_pages['loadingfailed'])
        self.notepage.set_current_page(self.progress_pages['main'])
        self.top_label.hide()

    # load sucess，加载系统更新信息页面，显示能够升级的项.
    def go_to_loadingsucceed_page(self, info={}):
        self.top_label.set_text(_("System Update"))
        self.top_label.show()
        self.loading_succeed_page(info=info)

        self.main_notebook.set_current_page(self.main_notebook_pages['loadingsucceed'])
        self.notepage.set_current_page(self.progress_pages['main'])

    def set_update_sensitive(self, status):
        try:
            self.system_update_obj.button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass
        try:
            self.app_button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass
        try:
            for tmp_app_obj in self.app_query:
                tmp_app_obj.button.set_sensitive(status)
        except Exception as e:
            pass
        finally:
            pass

    #根据返回的json，加载页面
    def loading_succeed_page(self, info={}):
        self.loadingsucceed_box = self.builder.get_object("loadingsucceed_box")
        self.loadingsucceed_box.set_name("loadingsucceed_box")
        self.loadingsucceed_box.set_halign(Gtk.Align.FILL)
        self.loadingsucceed_box.set_valign(Gtk.Align.START)
        self.loadingsucceed_grid = Gtk.Grid()

        i = 0
        separator = Gtk.HSeparator()
        separator.show()
        self.loadingsucceed_box.pack_end(self.loadingsucceed_grid, False, False, 0)
        self.loadingsucceed_grid.attach(separator, 0, i, 3, 1)
        i += 1

        # system upgrade
        upgrade_image_name = os.path.join( self.img_dir, "cdos.png")
        if ( "_upgrade" in info) and ( "new_ver" in info["_upgrade"]) and ( "new_release" in info["_upgrade"]) and ( "status" in info["_upgrade"]) and (info["_upgrade"]["status"]):
            upgrade_name = _("NFS Desktop OS Upgrade to %s") %str(info["_upgrade"]["new_ver"])
            upgrade_name_msg = str(info["_upgrade"]["new_ver"])
            detail_info = _("Your NFS Desktop OS can be upgraded to %s") %str(info["_upgrade"]["new_ver"])
            def upgrade_event_callback():
                if self.busy_flag == True:
                    warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
                    warning_dialog.set_label(_("NFS Desktop OS Update is working.\nTry it later."))
                    warning_dialog.show()
                    return
                # ToDo: do upgrade
                self.upgrade_flag = True
                self.set_update_sensitive(False)
                self.iface.can_dist_upgrade()
                self.updates_handler_json["upgrade"].hide_button()
                self.updates_handler_json["upgrade"].show_progress_box()
                self.updates_handler_json["upgrade"].progressbar.set_fraction(0)
                self.updates_handler_json["upgrade"].progressbar_info_label.set_text(_("Prepareing upgrade tools"))
        else:
            upgrade_name = _("Your NFS Desktop OS is the newest version")
            upgrade_name_msg = str(info["_upgrade"]["new_release"])
            detail_info = _("NFS Desktop OS version is: %s") %str(info["_upgrade"]["new_release"])
            upgrade_event_callback = None
        show_upgrade_detail = True
        system_upgrade_obj = Updates_Object(update_type="upgrade" ,image_name=upgrade_image_name, update_name=upgrade_name, update_name_msg=upgrade_name_msg, 
            show_detail=show_upgrade_detail, detail_info=detail_info, button_callback=upgrade_event_callback)
        self.updates_handler_json["upgrade"] = system_upgrade_obj
        self.loadingsucceed_grid.attach(system_upgrade_obj, 0, i, 3, 1)
        i += 1
        # if you have system updates
        if ("_deb_res" in info) and ("_patches" in info) and ("num" in info["_deb_res"]) and (len(info["_patches"])+info["_deb_res"]["num"] > 0):
            update_image_name = os.path.join( self.img_dir, "cdos_core.png")
            sys_up_name = _("NFS Desktop OS core updates")
            if len(info["_patches"])+info["_deb_res"]["num"] == 1:
                sys_up_msg = _("You have %d update") %(len(info["_patches"])+info["_deb_res"]["num"])
            else:
                sys_up_msg = _("You have %d updates") %(len(info["_patches"])+info["_deb_res"]["num"])
            detail_info = ""
            history_long_desc = ""
            if info["_deb_res"]["num"] > 0:
                if info["_deb_res"]["num"] == 1:
                    detail_info += _("You have %d core deb, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                    history_long_desc = _("Install %d core deb, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                else:
                    detail_info += _("You have %d core debs, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
                    detail_info += "\n"
                    for deb in info["_deb_res"]["list"]:
                        detail_info += "\n" + str(deb)
                    history_long_desc = _("Install %d core debs, %s") %(info["_deb_res"]["num"], str(info["_deb_res"]["size"]))
            else:
                info["_deb_res"]["list"] = []
            if len(info["_patches"]) > 0:  # if patches
                if info["_deb_res"]["num"] > 0:  # if patches and debs
                    detail_info +=  "\n"
                if len(info["_patches"]) == 1:
                    detail_info +=  _("%d patch may be installed") %len(info["_patches"])
                else:
                    detail_info +=  _("%d patches may be installed") %len(info["_patches"])
            for patch in info["_patches"]:
                detail_info +=  "\n" + "  " + str(patch) + ": " + str(info["_patches"][patch]["description"])
            def update_event_callback(ev_type, deb_list, obj, action):
                ev = {
                    "type":"system_update",
                    "ev_info": {
                        "debs": deb_list
                    },
                    "cancel_obj": obj
                }
                if action == "update":
                    GLib.idle_add(self.push_query, ev)
                else:
                    GLib.idle_add(self.pop_query, ev)

            self.system_update_obj = Updates_Object(update_type="update" ,image_name=update_image_name, update_name=sys_up_name, update_name_msg=sys_up_msg, 
                show_detail=True, detail_info=detail_info, size=info["_deb_res"]["size"], button_callback=update_event_callback, deb_list=info["_deb_res"]["list"], history_long_desc=history_long_desc)
            self.updates_handler_json["system"] = self.system_update_obj
            self.loadingsucceed_grid.attach(self.system_update_obj, 0, i, 3, 1)
            i += 1

        if len(info) > 3:
            self.app_title_grid = Gtk.Grid()
            self.app_label = Gtk.Label()#多少个应用更新文字
            self.app_label.set_xalign(0)
            alignment = Gtk.Alignment()
            alignment.set_hexpand(True)
            alignment.set_halign(Gtk.Align.FILL)
            self.app_button = Gtk.LinkButton()#全部更新按钮
            self.app_button.set_label(_("update all applications"))
            self.app_title_grid.attach(self.app_label, 0, 0, 1, 1)
            self.app_title_grid.attach(alignment, 1, 0, 1, 1)
            self.app_title_grid.attach(self.app_button, 2, 0, 1, 1)
            self.app_label.set_margin_top(34)
            self.app_label.set_name("top_label")#名字可能跟样式有关
            self.app_button.set_margin_top(34)
            self.app_button.set_name("detail_button")
            self.app_label.show()
            alignment.show()
            self.app_button.show()
            app_separator = Gtk.HSeparator()#分隔线.
            app_separator.show()
            self.app_title_grid.attach(app_separator, 0, 1, 3, 1)
            self.loadingsucceed_grid.attach(self.app_title_grid, 0, i, 3, 1)
            i += 1

            for app_info in info:
                if ((app_info == "_deb_res") or (app_info == "_patches") or (app_info == "_upgrade")):
                    continue
                try:
                    app_name = info[app_info]["AppName"]
                    app_msg = _("Version:")  + info[app_info]["Version"]
                    app_details = info[app_info]["LongDesc"] + "\n" + _("Version:")  + info[app_info]["Version"] + "\n" + _("Size:") + info[app_info]["size"]
                    icon = info[app_info]["Icon"]
                    def app_update_event_callback(ev_type, deb_list, obj, action):
                        ev = {
                            "type":"app_update",
                            "ev_info": {
                                "debs": deb_list
                            }
                        }
                        if action == "update":
                            GLib.idle_add(self.push_query, ev)
                        else:
                            GLib.idle_add(self.pop_query, ev)

                    app_obj = Updates_Object(update_type="app", image_name=icon, update_name=app_name, update_name_msg=app_msg, 
                        show_detail=True, detail_info=app_details, version=info[app_info]["Version"] ,size=info[app_info]["size"] ,button_callback=app_update_event_callback, deb_list=[info[app_info]["Package"], ])
                except Exception as e:
                    continue
                self.updates_handler_json["apps"][app_info] = app_obj
                self.app_query.append(app_obj)
                self.loadingsucceed_grid.attach(app_obj, 0, i, 3, 1)
                self.app_num = len(self.updates_handler_json["apps"])
                if self.app_num == 1:
                    self.app_label.set_text(_("%d application to be updated") %self.app_num)
                else:
                    self.app_label.set_text(_("%d applications to be updated") %self.app_num)
                if self.app_num > 0:
                    self.app_title_grid.show()

                app_obj.show()
                i += 1
            def on_app_button_clicked(tmp_arg):
                for tmp_app_obj in self.app_query:
                    if tmp_app_obj.ready == False:
                        tmp_app_obj.do_work()
            self.app_button.connect("clicked", on_app_button_clicked)

        if self.history_db is None:
            self.loadingsucceed_grid.show()
            return

        #更新历史.
        self.history_title_grid = Gtk.Grid()
        self.history_label = Gtk.Label()
        self.history_label.set_text(_("history in 30 days"))
        self.history_label.set_xalign(0)
        self.history_label.set_margin_end(8)
        self.alignment = Gtk.Alignment()
        self.alignment.set_hexpand(True)
        self.alignment.set_halign(Gtk.Align.FILL)
        self.alignment.set_margin_end(8)
        self.history_title_grid.attach(self.history_label, 0, 0, 1, 1)
        self.history_title_grid.attach(self.alignment, 1, 0, 1, 1)
        self.history_label.set_margin_top(34)
        self.history_label.set_name("top_label")
        self.history_separator = Gtk.HSeparator()
        self.alignment.set_margin_top(8)
        self.history_title_grid.attach(self.history_separator, 0, 1, 3, 1)
        self.loadingsucceed_grid.attach(self.history_title_grid, 0, i, 3, 1)
        i += 1
        self.history_inster = i
        try:
            history_infos = self.history_db.search(30)
            self.his_length = len(history_infos)

            for history_info in history_infos:
                try:
                    update_type = history_info["type"]
                    app_name = history_info["appname"]
                    version = history_info["version"]
                    size = history_info["size"]
                    icon = history_info["icon"]
                    shortdesc = history_info["shortdesc"]
                    longdesc = history_info["longdesc"]
                    if update_type == "systemupdate":
                        app_obj = Updates_Object(update_type="update", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                    elif update_type == "application":
                        app_obj = Updates_Object(update_type="app", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                except Exception as e:
                    logging.info("Insert history Exception:" + str(e))
                    continue
                self.loadingsucceed_grid.insert_row(i)
                self.loadingsucceed_grid.attach(app_obj, 0, i, 3, 1)
                app_obj.show()

        except Exception as e:
            logging.info("Insert history Exception:" + str(e))
            self.his_length = 0

        self.show_history_title()

        self.loadingsucceed_grid.show()

    def inster_history(self, update_type, image_name, update_name, update_name_msg, show_detail, detail_info):
        try:
            app_obj = Updates_Object(update_type=update_type, image_name=image_name, update_name=update_name, update_name_msg=update_name_msg, 
                show_detail=show_detail, detail_info=detail_info)
            self.loadingsucceed_grid.insert_row(self.history_inster)
            self.loadingsucceed_grid.attach(app_obj, 0, self.history_inster, 3, 1)
            app_obj.show()
            self.his_length += 1
            self.show_history_title()
        except Exception as e:
            pass

    def show_history_title(self):
        if self.his_length > 0:
            self.history_label.show()
            self.alignment.show()
            self.history_separator.show()
            self.history_title_grid.show()
        else:
            self.history_label.hide()
            self.alignment.hide()
            self.history_separator.hide()
            self.history_title_grid.hide()

    def on_setting_button_clicked(self, widget):
        self.go_to_settings_page()

    def init_setting_page(self):
        self.setting_label = self.builder.get_object("setting_label")
        self.setting_label.set_name("setting_label")
        self.setting_label.set_text(_("Select the method of checking and downloading updates for Fangde desktop operating system"))
        self.setting_label2 = self.builder.get_object("setting_label2")
        self.setting_label2.set_name("setting_label2")
        self.setting_label2.set_text(_("When the computer is connected to the Internet, the system will automatically check for important updates and remind you according to your settings."))
        self.time_setting_label = self.builder.get_object("time_setting_label")
        self.time_setting_label.set_name("time_setting_label")
        self.time_setting_label.set_text(_("Timing check time:"))
        # self.time_setting_label2 = self.builder.get_object("time_setting_label2")
        # self.time_setting_label2.set_name("time_setting_label2")
        # self.time_setting_label2.set_text(_("at"))
        # setting frequency
        self.updatefrequence_viewport = self.builder.get_object("updatefrequence_viewport")#更新频率下拉框，每天检查更新.
        self.updatefrequence_viewport.set_name("updatefrequence_viewport")
        self.dayfreq_viewport = self.builder.get_object("dayfreq_viewport")#每日，每周几下拉框.
        self.dayfreq_viewport.set_name("dayfreq_viewport")
        self.hourfreq_viewport = self.builder.get_object("hourfreq_viewport")#整点时间下拉框.
        self.hourfreq_viewport.set_name("hourfreq_viewport")

        self.updateFrequence_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.updateFrequenceArray = [_("Regularly check for updates (recommended)"), _("Never check for updates")]
        for item in self.updateFrequenceArray:
            self.updateFrequence_list_store.append([item])
        self.updatefrequence = self.builder.get_object("updatefrequence")
        self.updatefrequence.set_name('updatefrequence')
        self.updatefrequence.set_model(self.updateFrequence_list_store)
        self.updatefrequence.set_active(0)
        self.updatefrequence.connect("changed", self.on_changed_updatefrequence)#检测更新频率更改连接的函数.

        self.dayfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.dayfreqArray = [_("Everyday"), _("Every Monday"), _("Every Tuesday"), _("Every Wednesday"), _("Every Thursday"), _("Every Friday"), _("Every Saturday"), _("Every Sunday")]
        for item in self.dayfreqArray:
            self.dayfreq_list_store.append([item])
        self.dayfreq = self.builder.get_object("dayfreq")
        self.dayfreq.set_name('dayfreq')
        self.dayfreq.set_model(self.dayfreq_list_store)
        self.dayfreq.set_active(0)
        self.dayfreq.connect("changed", self.on_changed_dayfreq)#每天或者每周几设置更改后连接的函数.

        self.hourfreq_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        for i in range(24):
            self.hourfreq_list_store.append([str(i)+":00"])
        self.hourfreq = self.builder.get_object("hourfreq")
        self.hourfreq.set_name('hourfreq')
        self.hourfreq.set_model(self.hourfreq_list_store)
        self.hourfreq.set_active(0)
        self.hourfreq.connect("changed", self.on_changed_hourfreq)#每天几点检测更新设置更改后连接的函数.
        # dist-upgrade options
        self.distupgrade_label = self.builder.get_object("distupgrade_label")
        self.distupgrade_label.set_name("distupgrade_label")
        self.distupgrade_label.set_text(_("Download update method:"))
        self.distupgrade_list_store = Gtk.ListStore(GObject.TYPE_STRING)
        self.distupgrade_list_array = [_("Automatic"), _("Manual")]
        for item in self.distupgrade_list_array:
            self.distupgrade_list_store.append([item])
        self.distupgrade_list = self.builder.get_object("distupgrade_list")#系统升级下拉框选项.
        self.distupgrade_list.set_name('distupgrade_list')
        self.distupgrade_list.set_model(self.distupgrade_list_store)
        #self.distupgrade_list.set_active(0)
        self.distupgrade_list.connect('changed', self.on_changed_distupgrade)#系统升级设置更改连接的函数.
        self.distupgrade_note = self.builder.get_object("distupgrade_note")
        self.distupgrade_note.set_name("distupgrade_note")
        self.distupgrade_note.set_text(_("Note: if 'automatic' is selected, the system will automatically download updates when the computer is connected to the internet"))
        # confirm or cancel
        self.confirm_btn = self.builder.get_object('confirm')
        self.confirm_btn.set_name('confirm')
        self.confirm_btn.set_label(_('Confirm'))
        #self.confirm_btn.connect("button_press_event", self.on_confirm_press)
        self.confirm_btn.connect("clicked", self.on_confirm_click)
        self.cancel_btn = self.builder.get_object('cancel')
        self.cancel_btn.set_name('cancel')
        self.cancel_btn.set_label(_('Cancel'))
        self.cancel_btn.connect("clicked", self.on_cancel_click)
        self.confirm_btn.set_sensitive(False)

        self.__get_gio_setting()

    def __get_gio_setting(self):
        schema = self.schema
        dayfreq = schema.get_int("check-day")
        hourfreq = schema.get_int("check-hour")
        self.setDayFrequecy(dayfreq, hourfreq)
        autoinstall = schema.get_boolean("autoinstall")
        autodownload = schema.get_boolean("autodownload")
        willcheckupdate = schema.get_boolean("checkupdate")
        distupgrade_opts = schema.get_int("distupgrade-opts")
        self.setFrequecy(willcheckupdate, autodownload, autoinstall)
        self.set_distupgrade(distupgrade_opts)
        self.confirm_btn.set_sensitive(False)
        self.cancel_btn.set_sensitive(True)

    def setFrequecy(self, willcheckupdate, autodownload, autoinstall):
        frequecy = None
        if not willcheckupdate:
            frequecy = self.updateFrequenceArray[1]
        else:
            frequecy = self.updateFrequenceArray[0]

        iterator = self.updateFrequence_list_store.get_iter_first()
        while iterator:
            if self.updateFrequence_list_store[iterator][0] == frequecy:
                if iterator != self.updatefrequence.get_active_iter():
                    self.updatefrequence.set_active_iter(iterator)
                break
            iterator = self.updateFrequence_list_store.iter_next(iterator)
        self.set_updatefrequence()

    # def on_changed_updatefrequence(self, combobox):
    #设置是否每日检查更新（更新频率）.
    def set_updatefrequence(self):
        combobox = self.updatefrequence
        value = combobox.get_active()
        willcheckupdate = False
        if value == 0:
            willcheckupdate = True
            if not self.dayfreq.is_sensitive():
                self.dayfreq.set_sensitive(True)
                self.hourfreq.set_sensitive(True)
        else:
            willcheckupdate = False
            if self.dayfreq.is_sensitive():
                self.dayfreq.set_sensitive(False)#灰掉按钮
                self.hourfreq.set_sensitive(False)
            
        #修改gsettings中的配置值.
        self.schema.set_boolean("checkupdate", willcheckupdate)

    def setDayFrequecy(self, dayfreq, hourfreq):
        iterator = self.dayfreq_list_store.get_iter_first()
        while iterator:
            if self.dayfreq_list_store[iterator][0] == self.dayfreqArray[dayfreq]:
                if iterator != self.dayfreq.get_active_iter():
                    self.dayfreq.set_active_iter(iterator)
                break
            iterator = self.dayfreq_list_store.iter_next(iterator)

        iterator = self.hourfreq_list_store.get_iter_first()
        while iterator:
            if self.hourfreq_list_store[iterator][0] == (str(hourfreq)+":00"):
                if iterator != self.hourfreq.get_active_iter():
                    self.hourfreq.set_active_iter(iterator)
                break
            iterator = self.hourfreq_list_store.iter_next(iterator)

    # def on_changed_dayfreq(self, combobox):
    def set_dayfreq(self):
        combobox = self.dayfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        chosen_item = -1
        for i in range(len(self.dayfreqArray)):
            if self.dayfreqArray[i] == model[iterator][0]:
                chosen_item = i
                break
        if chosen_item >= 0:
            schema = self.schema
            schema.set_int("check-day", chosen_item)

    # def on_changed_hourfreq(self, combobox):
    def set_hourfreq(self):
        combobox = self.hourfreq
        model = combobox.get_model()
        iterator = combobox.get_active_iter()
        schema = self.schema
        schema.set_int("check-hour", int(model[iterator][0].split(':')[0]))

    #设置更新频率更改后，激活确定按钮.
    def on_changed_updatefrequence(self, combobox):
        if combobox.get_active() == 0:
            self.dayfreq.set_sensitive(True)
            self.hourfreq.set_sensitive(True)
        elif combobox.get_active() == 1:
            self.dayfreq.set_sensitive(False)
            self.hourfreq.set_sensitive(False)
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_dayfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def on_changed_hourfreq(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)
    
    def on_changed_distupgrade(self, combobox):
        self.confirm_btn.set_sensitive(True)
        self.cancel_btn.set_sensitive(True)

    def set_update_distupgrade(self):
        iterator = self.distupgrade_list.get_active()
        schema = self.schema
        schema.set_int("distupgrade-opts", iterator)

    def set_distupgrade(self, opts):
        self.distupgrade_list.set_active(opts)

    #执行press后，设置按钮颜色，返回主页.
    def on_confirm_click(self, button):
        try:
            self.set_updatefrequence()
            self.set_dayfreq()
            self.set_hourfreq()
            self.set_update_distupgrade()
        except Exception as e:
            logging.error(str(e))
        self.confirm_btn.set_sensitive(False)
        self.go_to_main_page()

    #取消按钮对应的函数，获取初始值，设置页面.有个问题？？设置页面的同时也设置了gsettings，没有必要，需要修改？？？
    def on_cancel_click(self, button):
        schema = self.schema
        dayfreq = schema.get_int("check-day")
        hourfreq = schema.get_int("check-hour")
        self.setDayFrequecy(dayfreq, hourfreq)
        autoinstall = schema.get_boolean("autoinstall")
        autodownload = schema.get_boolean("autodownload")
        willcheckupdate = schema.get_boolean("checkupdate")
        distupgrade_opts = schema.get_int("distupgrade-opts")
        self.setFrequecy(willcheckupdate, autodownload, autoinstall)
        self.set_distupgrade(distupgrade_opts)
        self.confirm_btn.set_sensitive(False)
        self.go_to_main_page()

    #new add
    def update_download(self, patch_name, patch_id):
        self.patch_name = patch_name
        self.patch_id = patch_id
        self.iface.download_patch(self.env, patch_name)

    def update_install(self, patch_name, patch_id):
        self.iface.install_patch(self.env, patch_name, patch_id)

    def update_import(self):
        self.iface.import_offline_patch(self.env)

    def update_backup(self, patch_name, patch_id):
        self.patch_name = patch_name
        self.patch_id = patch_id
        return self.iface.backup()

    def check_backup(self):
        return self.iface.check_backup_data_exists()

    def check_network_connection(self):
        # 无法检测404异常，仅用来检测是否有网络链接
        result = ""
        try: 
            urllib.request.urlopen(self.repo_url, timeout=3)
        except Exception as e:
            logging.error(str(e))
            return False
        try:
            result = urllib.request.urlopen(self.new_repo_url, timeout=3).read().decode()
        except Exception as e:
            logging.error(str(e))
            return False
        if not ("panda" in result or "tiger" in result):
            return False
        return True

    def copy_files_to_import_dir(self, list_file):
        dest_dir = "/tmp/offline-patch"
        if os.path.exists(dest_dir):
            shutil.rmtree(dest_dir, True)
        os.mkdir(dest_dir)
        try:
            for file in list_file:
                shutil.copy2(file, dest_dir)
        except Exception as e:
            print(str(e))
            return False
        return True
        
    def on_import_button_clicked(self, butten):
        print("on_import_button_clicked")
        if self.updates_check_obj.get_spinner_status():
            TipsDialog(_("Tips"), self.window,
                            _("Checking for updates, please wait..."), 
                            "",
                            callback_cancel=False)
            return
        if len(self.update_items):
            TipsDialog(_("Tips"), self.window,
                            _("unable to import update package!"), 
                            _("an online update package is detected. Please complete the online update first"),
                            callback_cancel=False)
            return

        iterable = [_("_Cancel"), Gtk.ResponseType.CANCEL, _("_Open"), Gtk.ResponseType.OK]
        dialog = Gtk.FileChooserDialog(_("select import update package"),
                                        self.window,
                                        Gtk.FileChooserAction.OPEN,
                                        iterable)
        dialog.set_select_multiple(True)
        filter = Gtk.FileFilter()
        filter.set_name("deb")
        filter.add_pattern ("*.deb")
        #filter.add_mime_type (".txt")
        dialog.add_filter(filter)

        def on_select_files_clicked(chooser, value):
            if value == Gtk.ResponseType.OK:
                filenames = chooser.get_filenames() 
                str_list_file = ""
                list_file = []
                for file in filenames:
                    filename = file[file.rindex("/") + 1 : len(file)]
                    list_file.append(filename)
                    str_list_file += filename
                    str_list_file += "\n"

                tips = TipsDialog(_("Tips"),
                                    chooser,
                                    _("Are you sure to import the following packages?"), 
                                    str_list_file)

                if tips.accept:
                    print("tips.accept")
                    self.update_tips = TipsDialog(_("Tips"), None,
                                        _("importing update package, please wait..."), 
                                        "",
                                        callback_ok=False,
                                        callback_cancel=False)
                    result = False
                    res, info = self.iface.verify_offline_patch(list_file)
                    #info = {"res":True,"_patches":{"patch_name":"xxxx","patch_id":10,"import_patch_list":["4.0-s006-nfspatch1","4.0-s006-nfspatch2"]},"_secure_patches": {"patch_name":"xxxx","patch_id":12,"import_patch_list":["4.0-s006-secure-nfspatch1","4.0-s006-secure-nfspatch2"]}}
                    if res:
                        if ("_patches" in info) and ("patch_name" in info["_patches"]) and ("patch_id" in info["_patches"]) and ("import_patch_list" in info["_patches"]): 
                            if ("_secure_patches" in info) and ("patch_name" in info["_secure_patches"]) and ("patch_id" in info["_secure_patches"]) and ("import_patch_list" in info["_secure_patches"]):
                                if self.copy_files_to_import_dir(filenames):
                                    self.import_info = info
                                    self.update_import()
                                    result = True

                    if not result:
                        self.update_tips.destroy()
                        TipsDialog(_("Tips"), chooser,
                                    _("update package import failed, please import again!"), 
                                    _("the update package validation is invalid or the update has completed"),
                                    callback_cancel=False)
                    chooser.destroy()
            elif value == Gtk.ResponseType.CANCEL:
                chooser.destroy()

        dialog.connect("response", on_select_files_clicked)
        dialog.show()

    def get_update_data(self):
        if not self.updates_check_obj.get_spinner_status():
            return False
        self.updates_check_obj.check_update_end()
        
        #加载更新信息
        info = {}
        try:
            info = self.iface.getAllUpdates(self.patch_url)
        except Exception as e:
            print(str(e))
            file = "/media/nfs_dev_sdb1/log/log_print2.json"
            if os.path.exists(file):
                with open(file, 'r') as content:
                    info = json.load(content)
            #return False

        self.refresh_updates(info)

        self.updates_check_obj.is_update = False
        self.updates_check_obj.name_msg_label.set_text(_("Last check for updates:") + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())))
        setLastCheckTime()

        backup_info = self.check_backup()
        if len(backup_info) == 3:
            self.refresh_backup(backup_info)

        return True

    def refresh_updates(self, info={}):
        self.row = 2
        if ("_patches" in info) and ("latestPatchName" in info["_patches"]) and (info["_patches"]["latestPatchName"] != "") and ("patchID" in info["_patches"]) and ("status" in info["_patches"]):
            self.update_item_append(info, "_patches")
        elif ("_patches" in info) and ("patch_name" in info["_patches"]) and (info["_patches"]["patch_name"] != "")and ("patch_id" in info["_patches"]) and ("import_patch_list" in info["_patches"]):
            self.update_item_import_append(info, "_patches")

        if ("_secure_patches" in info) and ("latestPatchName" in info["_secure_patches"]) and (info["_secure_patches"]["latestPatchName"] != "") and ("patchID" in info["_secure_patches"]) and ("status" in info["_secure_patches"]):
            self.update_item_append(info, "_secure_patches")
        elif ("_secure_patches" in info) and ("patch_name" in info["_secure_patches"]) and (info["_secure_patches"]["patch_name"] != "") and ("patch_id" in info["_secure_patches"]) and ("import_patch_list" in info["_secure_patches"]):
            self.update_item_import_append(info, "_secure_patches")

        if self.history_db is None:
            self.loadingsucceed_grid.show()
            return False

        #更新历史.
        self.history_title_grid = Gtk.Grid()
        self.history_label = Gtk.Label()
        self.history_label.set_text(_("history in 30 days"))
        self.history_label.set_xalign(0)
        self.history_label.set_margin_end(8)
        self.alignment = Gtk.Alignment()
        self.alignment.set_hexpand(True)
        self.alignment.set_halign(Gtk.Align.FILL)
        self.alignment.set_margin_end(8)
        self.history_title_grid.attach(self.history_label, 0, 0, 1, 1)
        self.history_title_grid.attach(self.alignment, 1, 0, 1, 1)
        self.history_label.set_margin_top(34)
        self.history_label.set_name("top_label")
        self.history_separator = Gtk.HSeparator()
        self.alignment.set_margin_top(8)
        self.history_title_grid.attach(self.history_separator, 0, 1, 3, 1)
        self.loadingsucceed_grid.attach(self.history_title_grid, 0, self.row, 4, 1)
        self.row += 1
        self.history_inster = self.row
        try:
            history_infos = self.history_db.search(30)
            self.his_length = len(history_infos)

            for history_info in history_infos:
                try:
                    update_type = history_info["type"]
                    app_name = history_info["appname"]
                    version = history_info["version"]
                    size = history_info["size"]
                    icon = history_info["icon"]
                    shortdesc = history_info["shortdesc"]
                    longdesc = history_info["longdesc"]
                    if update_type == "systemupdate":
                        app_obj = Updates_Object(update_type="update", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                    elif update_type == "application":
                        app_obj = Updates_Object(update_type="app", image_name=icon, update_name=app_name, update_name_msg=shortdesc, 
                            show_detail=True, detail_info=longdesc)
                except Exception as e:
                    logging.info("Insert history Exception:" + str(e))
                    continue
                self.loadingsucceed_grid.insert_row(self.row)
                self.loadingsucceed_grid.attach(app_obj, 0, self.row, 4, 1)
                print ("###history_infos####")
                app_obj.show()

        except Exception as e:
            logging.info("Insert history Exception:" + str(e))
            self.his_length = 0

        self.show_history_title()

    def refresh_backup(self, info=[]):
        def backspace_event_callback():
            result = os.popen("pkexec /usr/bin/cdosupdate-check")
            if result.read() != "cdos update frontend pkexec check succeeded\n":
                backspace_obj.button_origin_show()
                return
            if not self.working_items.count(backspace_obj):
                self.working_items.append(backspace_obj)
                if len(self.working_items) == 1:
                    self.is_rollback = True#区别回退和还原
                    if not self.iface.recover():
                        TipsDialog(_("Tips"), self.window,
                                _("failed to execute rollback"),
                                "",
                                self.update_item_restart,
                                False,
                                name_ok="Restart")
                        self.working_items[0].button_origin_show()
                        self.working_items.pop(0)
                        if len(self.working_items):
                            for item in self.working_items:
                                item.button_origin_show()
                                self.working_items.remove(item)
        
        backspace_obj = Backspace_Object("系统更新（" + info[0] + "）", info[1], info[2], backspace_event_callback)#"系统更新（G210-nfsupdate-1_1.0）",
        self.loadingsucceed_grid.attach(backspace_obj, 1, self.row, 3, 1)
        self.row += 1
       
    def handle_backup_recover_status_changed(self, msg):
        if ("type" in msg) and (msg['type'] == "silence_patch"):
            return

        if msg['action'] == 'backup':
            self.working_items[0].progressbar_show()
            if msg['status'] == 'inprogress':
                self.working_items[0].set_progress_bar(_("backup in progress, please wait..."), msg['percent'], False)
            elif msg['status'] == 'success':
                self.working_items[0].set_progress_bar(_("backup in progress, please wait..."), msg['percent'], False)
                self.update_install(self.patch_name, self.patch_id)
            elif msg['status'] == 'failed':
                TipsDialog(_("Tips"), self.window,
                                _("backup failed"), 
                                _("Backup Failed, this update package will be installed"),
                                callback_cancel=False)
                self.working_items[0].button_origin_show()
                self.working_items.pop(0)
                if len(self.working_items):
                    for item in self.working_items:
                        item.button_origin_show()
                        self.working_items.remove(item)

        if msg['action'] == 'recover':
            self.working_items[0].progressbar_show()
            if msg['status'] == 'inprogress':
                if self.is_rollback:
                    self.working_items[0].set_progress_bar(_("please wait while the system update is rolled back..."), msg['percent'], False)
                else:
                    self.working_items[0].set_progress_bar(_("installation failed, restoring changes, please wait..."), msg['percent'], False)
            elif msg['status'] == 'success':
                if self.is_rollback:
                    self.working_items[0].set_progress_bar(_("please wait while the system update is rolled back..."), msg['percent'], False)
                    TipsDialog(_("Tips"), self.window,
                                    _("fallback succeeded, please restart the computer!"), 
                                    _("please save all data before restarting"),
                                    self.update_item_restart,
                                    False,
                                    name_ok="Restart")
                else:
                    self.working_items[0].set_progress_bar(_("installation failed, restoring changes, please wait..."), msg['percent'], False)
                    TipsDialog(_("Tips"), self.window,
                                    _("failed to install update, please restart your computer!"), 
                                    _("after restart, the system will return to the pre installation state"),
                                    self.update_item_restart,
                                    False,
                                    name_ok="Restart")
                self.working_items[0].button_origin_show()
                self.working_items.pop(0)
                if len(self.working_items):
                    for item in self.working_items:
                        item.button_origin_show()
                        self.working_items.remove(item)
            elif msg['status'] == 'failed':
                if self.is_rollback:
                    TipsDialog(_("Tips"), self.window,
                                    _("fallback failed, please restart the computer!"), 
                                    _("error encountered while restoring"),
                                    self.update_item_restart,
                                    False,
                                    name_ok="Restart")
                else:
                    TipsDialog(_("Tips"), self.window,
                                    _("recover failed"), 
                                    _("error encountered while restoring"),
                                    self.update_item_restart,
                                    False,
                                    name_ok="Restart")
                self.working_items[0].button_origin_show()
                self.working_items.pop(0)
                if len(self.working_items):
                    for item in self.working_items:
                        item.button_origin_show()
                        self.working_items.remove(item)

    def handle_patchStatusChanged (self, msg):
        if ("type" in msg) and (msg['type'] == "silence_patch"):
            return
        
        if msg['action'] == 'import':
            if msg['status'] == 'inprogress':
                #self.working_items[0].set_progress_bar(_("imported"), msg['percent'])
                return
            elif msg['status'] == 'success':
                self.refresh_updates(self.import_info)
                if self.update_tips:
                    self.update_tips.destroy()
                #self.working_items[0].set_progress_bar(_("imported"), msg['percent'])
                #self.working_items[0].button.hide()
                #self.working_items[0].progressbar_box.show()
                #self.update_install(self.patch_name, self.patch_id)
            elif msg['status'] == 'failed':
                if self.update_tips:
                    self.update_tips.destroy()
                TipsDialog(_("Tips"), self.window,
                                _("system resource busy or network data error, please try again later!"), 
                                "")
 
        elif msg['action'] == 'download':
            if len(self.working_items) == 0 or (not self.working_items[0]):
                return
            self.working_items[0].progressbar_show(False)
            if msg['status'] == 'inprogress':
                self.working_items[0].set_progress_bar(_("downloaded"), msg['percent'])
            elif msg['status'] == 'success':
                self.working_items[0].set_progress_bar(_("downloaded"), msg['percent'])

                if not self.update_backup(self.patch_name, self.patch_id):
                    self.update_install(self.patch_name, self.patch_id)

            elif msg['status'] == 'failed':
                TipsDialog(_("Tips"), self.window,
                                _("system resource busy or network data error, please try again later!"), 
                                "")
                self.working_items[0].button_origin_show(False)
                self.working_items.pop(0)
        
                if len(self.working_items):
                    for item in self.working_items:
                        item.button_origin_show(False)
                        self.working_items.remove(item)
                    #self.working_items[0].update_exec()

        elif msg['action'] == 'install':
            if len(self.working_items) == 0 or (not self.working_items[0]):
                return
            self.working_items[0].progressbar_show()
            if msg['status'] == 'inprogress':
                self.working_items[0].set_progress_bar(_("updating, please wait..."), msg['percent'], False)
            elif msg['status'] == 'success':
                self.working_items[0].set_progress_bar("", msg['percent'])

                if self.working_items[0] in self.update_items:
                    index = self.update_items.index(self.working_items[0])
                    for item in range(index + 1, len(self.update_items)):
                        self.update_items[item].row -= 1
                    self.update_items.pop(index)
                    self.update_item_remove(self.working_items[0].row)
                self.working_items.pop(0)

                if len(self.working_items):
                    self.working_items[0].update_exec()
                else:
                    TipsDialog(_("Tips"), self.window,
                                _("the system update has been completed, please restart the computer!"), 
                                _("please save all data before restarting"),
                                self.update_item_restart,
                                True,
                                name_ok="Restart")
  
            elif msg['status'] == 'failed':
                self.working_items[0].set_progress_bar(_("installation failed, restoring changes, please wait..."), msg['percent'], False)
                self.is_rollback = False#区别回退和还原
                if not self.iface.recover():
                    TipsDialog(_("Tips"), self.window,
                                _("failed to install update, please restart your computer!"), 
                                "",
                                self.update_item_restart,
                                False,
                                name_ok="Restart")
                    self.working_items[0].button_origin_show()
                    self.working_items.pop(0)
                    if len(self.working_items):
                        for item in self.working_items:
                            item.button_origin_show()
                            self.working_items.remove(item)

    def update_item_append(self, info, category):
        update_type = "sys_patch"
        update_image_name = os.path.join(self.img_dir, "system_update.png")
        update_name = _("system update")
        if category == "_secure_patches":
            update_type = "secure_patch"
            update_image_name = os.path.join(self.img_dir, "security_update.png")
            update_name = _("security vulnerability update")

        update_name_msg = info[category]["desc"]
        detail_info = info[category]["desc"]

        is_download = False
        if info[category]["status"] == "download" :
            is_download = True

        def update_event_callback():
            if os.path.ismount('/opt/backup-recover'):            
                TipsDialog(_("Tips"), self.window,
                            _("the system provides an update protection mechanism. the current system will be backed up before the update is installed"),
                            _("the backup files are stored on the local disk"),
                            callback_cancel=False)
            if not self.working_items.count(update_item):
                self.working_items.append(update_item)
                if len(self.working_items) == 1:
                    if is_download:
                        if not self.update_backup(info[category]["latestPatchName"], info[category]["patchID"]):
                            self.update_install(info[category]["latestPatchName"], info[category]["patchID"])
                    else :
                        self.update_download(info[category]["latestPatchName"], info[category]["patchID"])
            else :
                #if self.working_items[0] == update_item:
                if is_download:
                    if not self.update_backup(info[category]["latestPatchName"], info[category]["patchID"]):
                        self.update_install(info[category]["latestPatchName"], info[category]["patchID"])
                else :
                    self.update_download(info[category]["latestPatchName"], info[category]["patchID"])
        
        update_item = Update_Items(update_type=update_type ,image_name=update_image_name, update_name=update_name, update_name_msg=update_name_msg,
            detail_info=detail_info, button_callback=update_event_callback, is_download=is_download)
        self.updates_handler_json["system"] = update_item
        
        self.loadingsucceed_grid.attach(update_item, 1, self.row, 3, 1)
        update_item.row = self.row
        self.row += 1
        self.update_items.append(update_item)

        if not self.updates_check_obj.is_update:
            self.updates_check_obj.name_label.set_text(_("check that your system has updates available!"))
            self.updates_check_obj.is_update = True
        #self.import_button.hide()
        #if not self.schema.get_int("distupgrade-opts"):
        #    update_item.update_wait()

    def update_item_import_append(self, info, category):
        type = "sys_patch"
        if category == "_secure_patches":
            type = "secure_patch"
        elif category == "_silence_patches":
            type = "silence_patch"

        update_type = "sys_patch"
        update_image_name = os.path.join(self.img_dir, "system_update.png")
        update_name = _("system update")
        if category == "_secure_patches":
            update_type = "secure_patch"
            update_image_name = os.path.join(self.img_dir, "security_update.png")
            update_name = _("security vulnerability update")

        update_name_msg = info[category]["patch_name"]
        detail_info = self.iface.get_offline_patch_desc(info[category]["patch_id"], type)

        def update_event_callback():
            if os.path.ismount('/opt/backup-recover'):
                TipsDialog(_("Tips"), self.window,
                            _("the system provides an update protection mechanism. the current system will be backed up before the update is installed"),
                            _("the backup files are stored on the local disk"),
                            callback_cancel=False)
            if not self.working_items.count(update_item):
                self.working_items.append(update_item)
                if len(self.working_items) == 1:
                    if not self.update_backup(info[category]["patch_name"], info[category]["patch_id"]):
                        self.update_install(info[category]["patch_name"], info[category]["patch_id"])
            else :
                #if self.working_items[0] == update_item:
                if not self.update_backup(info[category]["patch_name"], info[category]["patch_id"]):
                    self.update_install(info[category]["patch_name"], info[category]["patch_id"])
        
        is_download = True
        
        update_item = Update_Items(update_type=update_type ,image_name=update_image_name, update_name=update_name, update_name_msg=update_name_msg,
            detail_info=detail_info, button_callback=update_event_callback, is_download=is_download)
        self.updates_handler_json["system"] = update_item
        self.loadingsucceed_grid.attach(update_item, 1, self.row, 3, 1)
        update_item.row = self.row
        self.row += 1
        self.update_items.append(update_item)

        if not self.updates_check_obj.is_update:
            self.updates_check_obj.name_label.set_text(_("check that your system has updates available!"))
            self.updates_check_obj.is_update = True


    def update_item_restart(self, data):
        self.iface.sys_reboot()

    def update_item_remove(self, row):
        if self.loadingsucceed_grid:
            self.loadingsucceed_grid.remove_row(row)

    def update_item_reboot_remove(self, row):
        if self.loadingsucceed_grid:
            self.loadingsucceed_grid.remove_row(row)
        self.iface.sys_reboot()

class Update_Items(Gtk.Grid):
    def __init__(self, update_type, image_name, update_name, update_name_msg, detail_info="", button_callback=None, is_download=False):
        Gtk.Grid.__init__(self)

        self.set_size_request(-1, 84)
        self.button_callback = button_callback
        self.detail_status = False
        self.type = update_type
        self.ready = False
        self.visible = False
        self.row = 2
        self.detail_info = detail_info
        self.image_name = image_name
        self.update_name = update_name
        self.update_name_msg = update_name_msg
        self.default_image_name = "/usr/share/cdos-update/daemon/software_center/default.png"

        alignment = Gtk.Alignment()
        alignment.set_hexpand(True)
        alignment.set_halign(Gtk.Align.FILL)
        self.attach(alignment, 0, 0, 1, 2)

        #图标.
        image = Gtk.Image()
        image_wigth = image_height = 54
        if os.path.exists(image_name):
            pixbuf = Pixbuf.new_from_file_at_scale(image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        else:
            pixbuf = Pixbuf.new_from_file_at_scale(self.default_image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        image.set_from_pixbuf(pixbuf)
        image.set_margin_top(14)
        image.set_margin_right(30)
        self.attach(image, 1, 0, 1, 2)

        #名称
        name_label = Gtk.Label(update_name)
        name_label.set_name("update_name")
        name_label.set_margin_top(22)
        #name_label.set_margin_right(135)
        name_label.set_xalign(0)
        self.attach(name_label, 2, 0, 1, 1)

        #描述信息
        name_msg_label = Gtk.Label(update_name_msg[:20] + "...")
        name_msg_label.set_name("update_name_msg")
        name_msg_label.set_size_request(130, 4)
        name_msg_label.set_halign(Gtk.Align.START)
        name_msg_label.set_xalign(0)
        self.attach(name_msg_label, 2, 1, 1, 1)

        #详情按钮
        self.detail_button = Gtk.LinkButton()
        self.detail_button.set_size_request(100, 4)
        self.detail_button.set_margin_right(80)
        self.detail_button.set_label(_("details..."))
        self.detail_button.set_name("detail_button")
        self.detail_button.set_halign(Gtk.Align.START)
        self.detail_button.modify_text(Gtk.StateType.NORMAL, Gdk.Color(237*256, 16*256, 218*256))
        self.detail_button.connect("clicked", self.change_detail_status)
        self.attach(self.detail_button, 3, 1, 1, 1)

        #进度条
        self.progressbar_info_up = Gtk.Label()
        self.progressbar_info_up.set_text(_("waiting..."))
        self.progressbar = Gtk.ProgressBar()
        self.progressbar.set_size_request(300, 4)
        self.progressbar.set_margin_top(6)
        self.progressbar.set_margin_bottom(6)
        self.progressbar_info_down = Gtk.Label()
        self.progressbar_info_down.set_name("detail_label")
        self.progressbar_info_down.set_text(_("do not shut down or restart the computer"))

        self.progressbar_box = Gtk.VBox()
        self.progressbar_box.set_margin_top(25)
        self.progressbar_box.set_margin_left(80)
        self.progressbar_box.pack_start(self.progressbar_info_up, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar_info_down, False, False, 0)
        self.attach(self.progressbar_box, 4, 0, 1, 2)

        self.button = Gtk.Button()
        if is_download:
            self.button.set_label(_("Install Updates"))
        else :
            self.button.set_label(_("Download And Install Updates"))
        
        self.button.set_name("update_button")
        self.button.set_margin_top(25)
        self.button.set_margin_left(250)
        self.button.set_size_request(130, 36)
        self.button.set_halign(Gtk.Align.END)
        self.button.set_valign(Gtk.Align.START)
        self.attach(self.button, 4, 0, 1, 2)

        self.button.connect("clicked", self.backup_choice)

        #详情信息
        self.detail_label = Gtk.Label(detail_info)
        self.detail_label.set_name("detail_label")
        self.detail_label.set_xalign(0)
        self.detail_label.set_margin_left(12)
        self.attach(self.detail_label, 2, 2, 4, 1)

        alignment2 = Gtk.Alignment()
        alignment2.set_vexpand(True)
        alignment2.set_valign(Gtk.Align.FILL)
        alignment2.set_halign(Gtk.Align.END)
        alignment2.set_valign(Gtk.Align.START)
        self.attach(alignment2, 2, 3, 6, 1)
        self.separator = Gtk.HSeparator()#分隔线
        self.separator.set_margin_top(14)
        self.attach(self.separator, 2, 4, 6, 1)

        self.show_all()
        self.progressbar_box.hide()
        self.detail_label.hide()

    def change_detail_status(self, *argv):
        if self.detail_status == True:
            self.detail_label.hide()
            self.detail_status = False
            self.detail_button.set_label(_("details..."))
        else:
            self.detail_label.show()
            self.detail_status = True
            self.detail_button.set_label(_("hide details"))

    def set_progress_bar(self, prefix, num, show = True):
        try:
            self.progressbar.set_fraction(float(float(num)/100.0))
            if show:
                self.progressbar_info_up.set_text(prefix + str(int(num)) + "%")
            else :
                self.progressbar_info_up.set_text(prefix)
        except Exception as e:
            logging.error("set progessbar error:" + str(e))
            self.progressbar.set_fraction(0)

    def backup_choice(self, value):
        self.button_wait_show()
        if self.button_callback:
            self.button_callback()
        # schema = Gio.Settings(schema = "org.cdos.update")
        # if schema.get_boolean("backupprompt") == False:
        #     self.update_exec()
        #     return
        # win = BackupConfirmWin()
        # win.show()

    def update_wait(self):
        self.button_wait_show()
        if self.button_callback:
            self.button_callback()

    def update_exec(self):
        if self.button_callback:
            self.button_callback()

    def button_show(self):
        self.progressbar_box.hide()
        self.button.show()
        self.visible = False

    def progressbar_show(self, down=True):
        self.button.hide()
        self.progressbar_box.show()
        if down:
            self.progressbar_info_down.show()
        else:
            self.progressbar_info_down.hide()
        self.visible = True

    def button_origin_show(self, downloaded=True):
        #if not self.button.is_visible:
        self.button_show()
        self.button.set_name("update_button")
        if downloaded:
            self.button.set_label(_("Install Updates"))
        else:
            self.button.set_label(_("Download And Install Updates"))
        self.button.set_sensitive(True)

    def button_wait_show(self):
        if not self.button.is_visible:
            self.button_show()
        self.button.set_name("waiting_install_button")
        self.button.set_label(_("Install Wait"))
        self.button.set_sensitive(False)

class Updates_Check_Object(Gtk.Grid):
    def __init__(self, image_name, button_callback=None):
        Gtk.Grid.__init__(self)

        self.set_size_request(-1, 84)
        self.is_update = False
        self.button_callback = button_callback

        default_image_name = "/usr/share/cdos-update/daemon/software_center/default.png"

        image = Gtk.Image()
        image_wigth = image_height = 54
        if os.path.exists(image_name):
            pixbuf = Pixbuf.new_from_file_at_scale(image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        else:
            pixbuf = Pixbuf.new_from_file_at_scale(default_image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        image.set_from_pixbuf(pixbuf)
        image.set_margin_top(14)
        image.set_margin_right(30)
        self.attach(image, 0, 0, 1, 2)

        #名称
        self.name_label = Gtk.Label(_("your system is the latest version!"))
        self.name_label.set_name("update_name")
        self.name_label.set_margin_top(22)
        self.name_label.set_xalign(0)
        self.attach(self.name_label, 1, 0, 1, 1)

        #描述信息
        time_stamp = getLastCheckTime().getValue()
        markup = _("Last check for updates:") + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time_stamp))
        if not time_stamp:
            markup = _("update check not performed")
            #time_stamp = int(time.time())
        self.name_msg_label = Gtk.Label(markup)
        self.name_msg_label.set_name("update_name_msg")
        self.name_msg_label.set_xalign(0)
        self.attach(self.name_msg_label, 1, 1, 1, 1)

        alignment = Gtk.Alignment()
        alignment.set_hexpand(True)
        alignment.set_halign(Gtk.Align.FILL)
        self.attach(alignment, 2, 0, 1, 2)

        self.frame = Gtk.Frame()
        self.frame.set_name("check_update_button")
        self.frame.set_margin_top(25)
        self.frame.set_margin_left(250)
        self.frame.set_size_request(130, 36)
        self.frame.set_halign(Gtk.Align.END)
        self.frame.set_valign(Gtk.Align.START)
        self.frame.set_shadow_type(Gtk.ShadowType.NONE)
        self.attach(self.frame, 3, 0, 1, 2)

        self.spinner = Gtk.Spinner()
        self.spinner.set_margin_top(5)
        self.spinner.set_margin_bottom(5)
        self.frame.add(self.spinner)

        self.button = Gtk.Button()
        self.button.set_label(_("Check Update"))
        self.button.set_name("check_update_button")
        self.button.set_margin_top(25)
        self.button.set_margin_left(250)
        self.button.set_size_request(130, 36)
        self.button.set_halign(Gtk.Align.END)
        self.button.set_valign(Gtk.Align.START)
        self.button.connect("clicked", self.check_update_start)
        self.attach(self.button, 3, 0, 1, 2)

        alignment2 = Gtk.Alignment()
        alignment2.set_vexpand(True)
        alignment2.set_valign(Gtk.Align.FILL)
        alignment2.set_halign(Gtk.Align.END)
        alignment2.set_valign(Gtk.Align.START)
        self.attach(alignment2, 1, 3, 6, 1)
        self.separator = Gtk.HSeparator()#分隔线
        self.separator.set_margin_top(14)
        self.attach(self.separator, 1, 4, 6, 1)

        self.show_all()
        self.frame.hide()

    def check_update_start(self, button):
        if not self.get_spinner_status():
            self.button.hide()
            self.frame.show()
            self.spinner.start()
            GLib.timeout_add(100, self.button_callback)

    def check_update_end(self):
        if self.get_spinner_status():
            self.frame.hide()
            self.spinner.stop()
            self.button.show()

    def get_spinner_status(self):
        return self.spinner.get_property("active")

class TipsDialog(Gtk.Dialog):
    def __init__(self, name, parent, title, desc, callback_ok=True, callback_cancel=True, name_ok="ok", name_cancel="Cancel", data=None):
        Gtk.Dialog.__init__(self, name, parent)
        self.set_size_request(400, -1)
        self.set_name("note-viewport")
        self.set_modal(True)
        self.accept = False
    
        if callback_cancel:
            button = self.add_button (_(name_cancel), 0)
            button.set_name("cancel_button")
            button.set_size_request(80, 30)
            button.set_margin_top(20)
            button.set_margin_bottom(20)
            button.set_margin_left(20)
            #button.set_margin_right(10)

        if callback_ok:
            button = self.add_button (_(name_ok), 1)
            button.set_name("ok_button")
            button.set_size_request(80, 30)
            button.set_margin_top(20)
            button.set_margin_bottom(20)
            button.set_margin_left(10)
            button.set_margin_right(20)

        widget = self.get_content_area()

        #vbox = Gtk.VBox(2)
        #widget.add(vbox)

        label = Gtk.Label(title)
        #label.set_name("label_title")
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.START)
        label.set_margin_top(20)
        label.set_margin_left(20)
        label.set_margin_right(20)
        widget.add(label)

        label = Gtk.Label(desc)
        label.set_name("label_desc")
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.START)
        label.set_margin_top(20)
        label.set_margin_bottom(20)
        label.set_margin_left(20)
        label.set_margin_right(20)
        widget.add(label)

        self.show_all()

        if callback_ok or callback_cancel:
            result = self.run()

        if (callback_ok and result == 1):
            self.accept = True
            if callable(callback_ok):
                callback_ok(data)
            
        if (callback_cancel and result == 0):
            self.accept = False
            if callable(callback_cancel):
                callback_cancel(data)

        if callback_ok or callback_cancel:
            if (result == Gtk.ResponseType.DELETE_EVENT):
                self.accept = False
                if callable(callback_cancel):
                    callback_cancel(data)
            self.destroy()

class Updates_Object(Gtk.Grid):
    def __init__(self, update_type, image_name, update_name, update_name_msg, show_detail=False, detail_info="", version="", size="", button_callback=None, deb_list=[], history_long_desc=""):
        Gtk.Grid.__init__(self)

        self.set_size_request(-1, 84)
        self.button_callback = button_callback
        self.detail_status = False
        self.type = update_type
        self.deb_list = deb_list
        self.ready = False
        self.working = False
        self.detail_info = detail_info
        self.image_name = image_name
        self.update_name = update_name
        self.update_name_msg = update_name_msg
        self.history_long_desc = history_long_desc
        self.version = version
        self.size = size
        self.default_image_name = "/usr/share/cdos-update/daemon/software_center/default.png"

        self.init_object(image_name=image_name, update_name=update_name, update_name_msg=update_name_msg, show_detail=show_detail, detail_info=detail_info,button_callback=button_callback)
        self.show_updates_info( show_detail, button_callback)

    def init_object(self,image_name, update_name, update_name_msg, show_detail, detail_info, button_callback):
        #图标.
        image = Gtk.Image()
        image_wigth = image_height = 54
        if os.path.exists(image_name):
            pixbuf = Pixbuf.new_from_file_at_scale(image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        else:
            pixbuf = Pixbuf.new_from_file_at_scale(self.default_image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        image.set_from_pixbuf(pixbuf)
        image.set_margin_top(14)
        image.set_margin_right(30)
        self.attach(image, 0, 0, 1, 2)

        #名称
        name_label = Gtk.Label(update_name)
        name_label.set_name("update_name")
        name_label.set_margin_top(22)
        name_label.set_xalign(0)
        self.attach(name_label, 1, 0, 2, 1)

        #描述信息
        name_msg_label = Gtk.Label(update_name_msg[:19] + "...")
        name_msg_label.set_name("update_name_msg")
        name_msg_label.set_xalign(0)
        self.attach(name_msg_label, 1, 1, 1, 1)

        #详情按钮
        self.detail_button = Gtk.LinkButton()
        self.detail_button.set_label(_("details..."))
        self.detail_button.set_name("detail_button")
        self.detail_button.set_halign(Gtk.Align.START)
        self.detail_button.modify_text(Gtk.StateType.NORMAL, Gdk.Color(237*256, 16*256, 218*256))
        self.detail_button.connect("clicked", self.change_detail_status)
        self.attach(self.detail_button, 2, 1, 1, 1)

        alignment = Gtk.Alignment()
        alignment.set_hexpand(True)
        alignment.set_halign(Gtk.Align.FILL)
        self.attach(alignment, 3, 0, 1, 2)

        #进度条
        self.progressbar_info_label = Gtk.Label()
        self.progressbar_info_label.set_text(_("waiting..."))
        self.progressbar = Gtk.ProgressBar()
        self.progressbar.set_size_request(380,4)
        self.progressbar.set_margin_top(4)
        self.progressbar_box = Gtk.VBox()
        self.progressbar_box.set_margin_top(30)
        self.progressbar_box.pack_start(self.progressbar_info_label, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar, False, False, 0)
        self.attach(self.progressbar_box, 4, 0, 1, 2)

        self.button = Gtk.Button()
        self.button.set_label(_("Update"))
        self.button.set_name("update_button")
        self.button.set_margin_top(25)
        self.button.set_margin_left(54)
        self.button.set_size_request(100, 32)
        self.button.set_halign(Gtk.Align.END)
        self.button.set_valign(Gtk.Align.START)
        self.attach(self.button, 5, 0, 1, 2)

        self.button.connect("clicked", self.backup_choice)

        #详情信息
        self.detail_label = Gtk.Label(detail_info)
        self.detail_label.set_name("detail_label")
        self.detail_label.set_xalign(0)
        self.detail_label.set_margin_left(12)
        self.attach(self.detail_label, 1, 2, 4, 1)

        alignment2 = Gtk.Alignment()
        alignment2.set_vexpand(True)
        alignment2.set_valign(Gtk.Align.FILL)
        alignment2.set_halign(Gtk.Align.END)
        alignment2.set_valign(Gtk.Align.START)
        self.attach(alignment2, 0, 3, 6, 1)
        self.separator = Gtk.HSeparator()#分隔线
        self.separator.set_margin_top(14)
        self.attach(self.separator, 1, 4, 6, 1)

    def change_detail_status(self, *argv):
        if self.detail_status == True:
            self.detail_label.hide()
            self.detail_status = False
            self.detail_button.set_label(_("details..."))
        else:
            self.detail_label.show()
            self.detail_status = True
            self.detail_button.set_label(_("hide details"))

    def hide_button(self):
        self.button.set_label("")
        self.button.set_sensitive(False)
        self.button.hide()
        self.progressbar_box.set_margin_right(154)
        if not self.type == "upgrade":
            self.show_progress_box()

    def finish(self):
        for widget in self.get_children():
            widget.destroy()
        self.destroy()

    def set_progress_bar(self, num):
        try:
            self.progressbar_info_label.set_text( str(int(num)) + "%")
            self.progressbar.set_fraction( float(float(num)/100.0))
        except Exception as e:
            logging.error("set progessbar error:" + str(e))
            self.progressbar.set_fraction(0)

    def do_work(self, *argc):
        self.ready = True
        self.button.set_label(_("Cancel"))
        self.button_callback(self.type, self.deb_list, self, action="update")
        self.show_progress_box()
    def cancel_func(self):
        self.ready = False
        self.button.set_label(_("Update"))
        self.hide_progress_box()

    def backup_confirm(self, *argv):
        intern_dbus_name = "org.cdos.update" 
        intern_dbus_path = "/org/cdos/update"
        intern_dbus_iface = "org.cdos.update"
        self.system_bus = dbus.SystemBus()
        self.remote_obj = self.system_bus.get_object( intern_dbus_name, intern_dbus_path )
        self.iface = dbus.Interface( self.remote_obj, intern_dbus_iface)
        res = self.iface.switch_backup_recover()
        logging.info('switch_backup_recover result: %d' % (res))
        if res != 0:
            warning_dialog = WarningDialog(self, uidir, img_dir)
            warning_dialog.set_label(_("Restart and backup failed. Please check whether the backup-recover system is installed"))
            warning_dialog.show()


    def backup_choice(self, *argv):
        schema = Gio.Settings(schema = "org.cdos.update")
        if schema.get_boolean("backupprompt") == False:
            self.update_exec()
            return

        win = BackupConfirmWin()
        win.show()
        self.backup_button_choice = win.getchoice()
        if self.backup_button_choice == 'confirm':
            self.backup_confirm()
        elif self.backup_button_choice == 'continue':
            self.update_exec()
        else:
            return 

    def update_exec(self, *argv):
        if self.type == "upgrade":
            self.button_callback()
            return
        else:
            self.show_progress_box()
        if self.button_callback is not None:
            if self.ready == False:
                self.ready = True
                self.button.set_label(_("Cancel"))
                self.button_callback(self.type, self.deb_list, self, action="update")
                self.button.hide()
                self.progressbar_box.set_margin_right(154)
                self.show_progress_box()
            else:
                #self.ready = False
                #self.button.set_label(_("Update"))
                self.button_callback(self.type, self.deb_list, self, action="cancel")
                #self.hide_progress_box()

    def finish_exec(self):
        self.ready = False
        self.show_button()
        self.progressbar.set_fraction( 0 )
        self.progressbar_info_label.set_text(_("waiting..."))
        self.hide_progress_box()

    def show_updates_info(self, show_detail, button_callback):
        self.show_all()
        self.hide_progress_box()
        self.detail_label.hide()
        if show_detail == False:
            self.detail_button.hide()
        if button_callback is None:
            self.button.hide()

    def set_button_label(self, label_info):
        self.button.set_label(_(label_info))

    def show_button(self):
        self.button.set_label(_("Update"))
        self.button.set_sensitive(True)
        self.button.show()
        self.progressbar_box.set_margin_right(0)

    def show_progress_box(self):
        self.progressbar_box.show()

    def hide_progress_box(self):
        self.progressbar_box.hide()

    def set_progress_info(self, info):
        self.progressbar_info_label.set_text(_(info))

class Backspace_Object(Gtk.Grid):
    def __init__(self, update_title, update_dec, update_date, button_callback=None):
        Gtk.Grid.__init__(self)
        
        self.box_h = Gtk.HBox()
        self.box_h.set_name("backspace_object")
        self.last_update_is_show = False
        self.row = 3
        self.button_callback = button_callback
        self.update_title = update_title
        self.update_dec = update_dec
        self.update_date = update_date
        alignment = Gtk.Alignment()
        alignment.set_hexpand(True)
        alignment.set_halign(Gtk.Align.FILL)
        self.attach(alignment, 0, 0, 1, 2)

        self.last_update_button = Gtk.Button()
        self.last_update_button.set_label(_("Last Update ＞"))
        self.last_update_button.set_name("last_update_button")
        self.last_update_button.set_halign(Gtk.Align.START)
        self.last_update_button.connect("clicked", self.last_update_callback)
        
        self.attach(self.last_update_button, 1, 0, 2, 1)
        self.attach(self.box_h, 1, 1, 2, 1)

        #详情信息
        self.last_update_title = Gtk.Label(update_title)
        self.last_update_title.set_name("update_name")
        self.last_update_title.set_xalign(0)

        self.last_update_des = Gtk.Label(update_dec[:21] + "...")
        self.last_update_des.set_name("detail_label")
        self.last_update_des.set_xalign(0)
        
        self.last_update_date = Gtk.Label(update_date)
        self.last_update_date.set_name("detail_label")
        self.last_update_date.set_xalign(0)
        
        self.info_box = Gtk.VBox()
        self.info_box.set_margin_end(15)
        self.info_box.set_spacing(10)
        self.info_box.pack_start(self.last_update_title, False, False, 0)
        self.info_box.pack_start(self.last_update_des, False, False, 0)
        self.info_box.pack_start(self.last_update_date, False, False, 0)
        self.info_box.set_valign(Gtk.Align.CENTER)
        self.box_h.pack_start(self.info_box, True, True, 0)
        
        #进度条
        self.progressbar_info_up = Gtk.Label()
        self.progressbar_info_up.set_text(_("waiting..."))
        self.progressbar = Gtk.ProgressBar()
        self.progressbar.set_size_request(300, 4)
        self.progressbar.set_margin_top(6)
        self.progressbar.set_margin_bottom(6)
        self.progressbar_info_down = Gtk.Label()
        self.progressbar_info_down.set_name("detail_label")
        self.progressbar_info_down.set_text(_("do not shut down or restart the computer"))

        self.progressbar_box = Gtk.VBox()
        self.progressbar_box.set_margin_start(40)
        self.progressbar_box.pack_start(self.progressbar_info_up, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar, False, False, 0)
        self.progressbar_box.pack_start(self.progressbar_info_down, False, False, 0)
        self.progressbar_box.set_valign(Gtk.Align.CENTER)
        self.box_h.pack_end(self.progressbar_box, False, False, 0)

        self.button = Gtk.Button()
        self.button.set_label(_("rollback"))
        self.button.set_name("update_button")
        self.button.set_margin_start(210)
        self.button.set_size_request(130, 36)
        self.button.set_valign(Gtk.Align.CENTER)
        self.button.connect("clicked", self.backup_choice)
        self.box_h.pack_end(self.button, False, False, 0)
        
        self.show_all()
        self.box_h.hide()
        self.progressbar_box.hide()
        #self.button.hide()

    def last_update_callback(self, button):
        if self.last_update_is_show:
            self.last_update_button.set_label(_("Last Update ＞"))
            self.last_update_is_show = False
        else:
            self.last_update_button.set_label(_("Last Update ∨"))
            self.last_update_is_show = True
        self.box_h.set_visible(self.last_update_is_show)

    def set_progress_bar(self, prefix, num, show = True):
        try:
            self.progressbar.set_fraction(float(float(num)/100.0))
            if show:
                self.progressbar_info_up.set_text(prefix + str(int(num)) + "%")
            else :
                self.progressbar_info_up.set_text(prefix)
        except Exception as e:
            logging.error("set progessbar error:" + str(e))
            self.progressbar.set_fraction(0)

    def backup_choice(self, value):
        self.button_wait_show()
        if self.button_callback:
            self.button_callback()

    def update_exec(self):
        if self.button_callback:
            self.button_callback()

    def button_show(self):
        self.progressbar_box.hide()
        self.button.show()
        self.visible = False

    def progressbar_show(self, down=True):
        self.button.hide()
        self.progressbar_box.show()
        if down:
            self.progressbar_info_down.show()
        else:
            self.progressbar_info_down.hide()
        self.visible = True

    def button_origin_show(self, downloaded=True):
        #if not self.button.is_visible:
        self.button_show()
        self.button.set_name("update_button")
        self.button.set_label(_("rollback"))
        self.button.set_sensitive(True)

    def button_wait_show(self):
        if not self.button.is_visible:
            self.button_show()
        self.button.set_name(_("waiting_install_button"))
        self.button.set_label(_("wait for rollback"))
        self.button.set_sensitive(False)

global intern_dbus_name
global intern_dbus_path
global intern_dbus_iface
intern_dbus_name = "org.cdos.update.ui"
intern_dbus_path = "/org/cdos/update/ui"
intern_dbus_iface =  "org.cdos.update.ui"

class OneInstance(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(intern_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, intern_dbus_path)

    @dbus.service.method(intern_dbus_iface, in_signature='', out_signature='b')
    def show(self):
        return True

def getRegisterStatus():
    try:
        system_bus = dbus.SystemBus()
        registerObj = system_bus.get_object('com.nfs.register', '/')
        iface = dbus.Interface(registerObj, dbus_interface='com.nfs.register')
        registerInfo = iface.get_dbus_method("getRegisterStatus", dbus_interface=None)
        return registerInfo('os', '')
    except Exception as e:
        logging.error("getRegisterStatus:" + str(e))
        return None

if __name__ == "__main__":
    try:
        dbml = DBusGMainLoop(set_as_default=True)
        dbus.set_default_main_loop(dbml)
        bus = dbus.SessionBus()
        remote_object_2 = bus.get_object(intern_dbus_name, intern_dbus_path )
        iface = dbus.Interface(remote_object_2, intern_dbus_iface)
        iface.show()
        exit(0)
    except Exception as e:
        oneinstance = OneInstance()

    iface_dbus_license = License_dbusInterface()
    licenseState = iface_dbus_license.getLicenseState()
    if licenseState == 2 or licenseState == 4: #激活状态为授权已过期和试用期已过期，弹出激活窗口，并退出
        os.system("notify-send -i 'cdos-update' '%s' '%s'" % (_('NFS Desktop OS Update'), _('Your system has not been activated yet. Please activate it as soon as possible to avoid affecting the complete system functionality and operating experience.')))
        iface_dbus_license.showLicenseActivator()
        exit(0)

    img_dir = "/usr/share/cdos-update/frontend/img"
    uidir = "/usr/share/cdos-update/frontend/ui"
    # registerResult = getRegisterStatus()
    # if registerResult == None:
    #     warning_dialog = WarningDialog(None, uidir, img_dir, True)
    #     warning_dialog.set_locale_label("Get registration information error.")
    #     warning_dialog.show()
    # elif registerResult[0] == 1:
    #     if registerResult[1] > 0:
    #         check = CheckMainwindow(img_dir, uidir)
    #         check.go_to_loading_page()
    #         check.show()
    #         GLib.idle_add(check.go_to_updates_page)
    #     else:
    #         warning_dialog = WarningDialog(None, uidir, img_dir, True)
    #         warning_dialog.set_locale_label("System registration has expired( %s ). Please re-register.", registerResult[2])
    #         warning_dialog.show()
    # else:
    #     warning_dialog = WarningDialog(None, uidir, img_dir, True)
    #     warning_dialog.set_locale_label("The system has not been registered yet.\nPlease complete registration first.")
    #     warning_dialog.show()
    check = CheckMainwindow(img_dir, uidir)

    #check.go_to_loading_page()
    check.show()

    #check.update_cache()

    #GLib.idle_add(check.go_to_updates_page)

    Gtk.main()


