#!/usr/bin/env python
# -*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
import dbus
import urllib.request, urllib.parse, urllib.error
import sys
import configparser
import gettext

from subprocess import Popen, PIPE
from gi.repository import Gtk, Gdk

gettext.bindtextdomain('cdosupdate', "/usr/share/locale/")
gettext.textdomain('cdosupdate')
_ = gettext.gettext

global SERVER
global uidir
uidir = sys.path[0] + "/ui"

origin_manifest = {}
new_manifest = {}
cur_manifest = {}

def get_server_url():
    ip = "repos.01os.org"
    global SERVER
    SERVER = ip + "/data/cdos-update/upgrade/cdos-upgrade"

def reboot():
    bus = dbus.SystemBus()
    bus_obj = bus.get_object("org.cdosupgrade","/org/cdosupgrade")
    iface = dbus.Interface(bus_obj, "org.cdosupgrade")
    try:
        iface.cdos_reboot()
    except dbus.exceptions.DBusException:
        print("reboot exception")
        sys.exit(1)
        
def reboot_dialog():
    msg = _("The upgrade has been completed, whether to restart?")
    dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK_CANCEL, msg)
    rc = dialog.run()
    if rc == Gtk.ResponseType.CANCEL or rc == Gtk.ResponseType.DELETE_EVENT:
        dialog.destroy()
    else:
        reboot()

def compare_version(cur_release, new_release):
    try:
        cur_release_date = cur_release.split('-')[2]
        new_release_date = new_release.split('-')[2]
    except:
        return 2
    if cur_release_date.isdigit() and new_release_date.isdigit():
        if new_release_date > cur_release_date:
            return 1
        else:
            return 0
    else:
        return 2
    
def get_upgrade_configfile(new_release):
    server_path = "https://%s/%s/" %(SERVER, new_release)
    local_path =os.path.join(os.getenv("HOME"), ".config/cdosupgrade/")
    grub_efimode_remove_list = ["grub-pc", "grub-pc-bin", "grub-gfxpayload-lists"]
    grub_efimode_add_list = ["mokutil", "grub-efi-amd64", "efibootmgr", "shim", "shim-signed", "grub-efi-amd64-bin", "grub-efi-amd64-signed"]
    if not os.path.exists(local_path):
        os.makedirs(local_path,mode=0o755)
    try:
        urllib.request.urlretrieve(server_path+"filesystem.manifest",local_path+"filesystem.manifest")
        urllib.request.urlretrieve(server_path+"official-package-repositories.list",local_path+"official-package-repositories.list")
        urllib.request.urlretrieve(server_path+"filesystem.manifest-remove",local_path+"filesystem.manifest-remove")
        urllib.request.urlretrieve(server_path+"os-release",local_path+"os-release")
        urllib.request.urlretrieve(server_path+"lsb-release",local_path+"lsb-release")
        urllib.request.urlretrieve(server_path+"fix_bug.py",local_path+"fix_bug.py")
    except:
        return False
    origin_manifest_path = ("/var/log/installer/filesystem.manifest")
    if os.path.exists(origin_manifest_path):
        with open(origin_manifest_path) as f:
            for line in f.readlines():
                result = line.strip("\n").split()
                if len(result) >= 2:
                    key = result[0]
                    value = result[1]
                    origin_manifest[key] = value
    cur_manifest_path = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/current.manifest")
    os.system("dpkg-query -W --showformat='${Package} ${Version}\n' > %s" % cur_manifest_path)
    with open(cur_manifest_path) as f:
        for line in f.readlines():
            result = line.strip("\n").split()
            if len(result) >= 2:
                 key = result[0]
                 value = result[1]
                 cur_manifest[key] = value
    if os.path.exists("/sys/firmware/efi"):
        for package in grub_efimode_add_list:
            new_manifest[package] = "0.0.1"
    new_manifest_path = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/filesystem.manifest")
    with open(new_manifest_path) as f:
        for line in f.readlines():
            result = line.strip("\n").split()
            if len(result) >= 2:
                 key = result[0]
                 value = result[1]
                 if os.path.exists("/sys/firmware/efi") and key in grub_efimode_remove_list:
                     continue
                 new_manifest[key] = value
    custom_pkgs = []
    nhold_pkgs = []
    upgrade_pkgs = []
    if os.path.exists(origin_manifest_path):        
        custom_pkgs = set(cur_manifest.keys()) - set(origin_manifest.keys())
        nhold_pkgs = set(origin_manifest.keys()) - set(new_manifest.keys()) - custom_pkgs
    new_pkgs = set(new_manifest.keys()) - set(cur_manifest.keys())
    upgrade_pkgs = set(new_manifest.keys()) - new_pkgs
    upgrade_pkgs_version = []
    upgrade_manifest_path = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/upgrade_pkgs")
    for package in upgrade_pkgs:
        if cur_manifest[package] == new_manifest[package]:
            continue
        else:
            item = "%s %s" %(package, new_manifest[package])
            upgrade_pkgs_version.append(item)
    with open(upgrade_manifest_path, 'w') as f:
        f.write('\n'.join(upgrade_pkgs_version))
    install_pkgs_list = []
    install_manifest_path = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/new_pkgs")
    for package in new_pkgs:
        item = "%s %s" %(package, new_manifest[package])
        install_pkgs_list.append(item)
    with open(install_manifest_path, 'w') as f:
        f.write('\n'.join(install_pkgs_list))
    return True

def update_system(backup_flag):
    new_repository_list_path = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/official-package-repositories.list")
    local_path = os.getenv("HOME")
    cmd = ["/usr/bin/cdos-release-upgrade-root", "%s" % new_repository_list_path, "%s" % local_path, "%s" % backup_flag]
    comnd = Popen(' '.join(cmd), shell=True)
    returnCode = comnd.wait()
    
def check_need_upgrade():
    get_server_url()
    new_release_file = "https://%s/upgrade.cfg" % SERVER
    local_conf_dir = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/")
    if not os.path.exists(local_conf_dir):
        os.mkdir(local_conf_dir)
    local_conf_file = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/upgrade.cfg")
    conf = configparser.ConfigParser(defaults=None)
    try:
        urllib.request.urlretrieve(new_release_file,local_conf_file)
        conf.read(local_conf_file)
    except:
        return 3
    new_release = conf.get("lastest", "OS_VERSION")
    with open ("/etc/os-release") as f:
        for line in f.readlines():
            if line.startswith("VERSION="):
                cur_release = eval(line.split("=")[-1].strip("\n"))
    return compare_version(cur_release, new_release)

def get_current_and_new_version():
    with open ("/etc/os-release") as f:
        for line in f.readlines():
            if line.startswith("VERSION="):
                cur_release = eval(line.split("=")[-1].strip("\n"))
    new_conf_file = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/os-release")
    local_conf_file = os.path.join(os.getenv("HOME"), ".config/cdosupgrade/upgrade.cfg")
    conf = configparser.ConfigParser(defaults=None)
    conf.read(local_conf_file)
    new_release = conf.get("lastest", "OS_VERSION")
    return (cur_release, new_release)


class UpgradeDialog():
    def __init__(self, new_release):
        css_provider = Gtk.CssProvider()
        css_provider.load_from_data(b'''\
        #label2 {
             color: #888888;
             font-size: 12px;
        }
        #label1 {
             color: #333333;
             font-size: 14px;
        }
        #button1 {
            -GtkWidget-focus-padding: 0;
            -GtkWidget-focus-line-width: 0;
            -GtkButton-inner-border: 0;

            background: #dddddd;
            color: #333333;
            box-shadow: none;
            font-weight: normal;
            font-size: 12px;
            border: none;
            border-radius: 0;
            border-color: #dddddd;
            border-width: 1px;
            border-style: solid;        
        }
        #button2 {
            -GtkWidget-focus-padding: 0;
            -GtkWidget-focus-line-width: 0;
            -GtkButton-inner-border: 0;

            background: #44A6E5;
            color: #FFFFFF;
            box-shadow: none;
            font-weight: normal;
            font-size: 12px;
            border: none;
            border-radius: 0;
            border-color: #44A6E4;
            border-width: 1px;
            border-style: solid;
        }
        #button1:active,
        #button1:active:hover {
            background: #c8c8c8;
            color: #333333;
            box-shadow: 0px 0px 0px 0px;        
        }
        #button2:active,
        #button2:active:hover {
            background: #3C93D3;
            color: #FFFFFF;
            box-shadow: 0px 0px 0px 0px;
        }

        #button2:focus,
        #button2:hover:focus,
        #button2:active:focus {
            border-color: shade(#44A6E4, 0.8);
            box-shadow: 0px 0px 0px 0px;
        }

        #button1:hover {
            background: #e7e7e7;
            color: #333333;
            box-shadow: 0px 0px 0px 0px;
        }
        #button2:hover {
            background: #48B2ED;
            color: #FFFFFF;
            box-shadow: 0px 0px 0px 0px;
        }
        ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)        
        self.builder = Gtk.Builder()
        self.builder.add_from_file(uidir + "/upgrade.ui")
        
        self.window = self.builder.get_object("dialog1")
        self.window.set_title(_("NFS Desktop OS Upgrade"))
              
        self.label_1 = self.builder.get_object("label1")
        self.label_1.set_name("label1")
        self.label_1.set_text(_("NFS operating system can be upgrade to new version %s, you can backup your system before upgrade to prevent data loss\nIt is recommended upgrade OS after you reboot your computer if you istall core updates recently") %new_release)
        self.label_2 = self.builder.get_object("label2")
        self.label_2.set_name("label2")
        self.label_2.set_text(_("after backup you can recover your old system for livecd mode,  The restore steps are as follows:\n"
	"1、Enter livecd, enter the home directory, find the drive you want to restore, double click access\n"
	"2、Open the terminal and enter the newly mounted hard disk directory\n"
	"3、Find the backup.tgz file in that directory and execute the command：sudo tar xvpfz backup.tgz -C ./\n"
	"4、Reboot into the restore system")) 
        self.button_1 = self.builder.get_object("button1")
        self.button_1.set_name("button1")
        self.button_1.set_label(_("backup and upgrade"))
        self.button_1.connect("clicked", self.on_backup_upgrade_click)
        self.button_2 = self.builder.get_object("button2")
        self.button_2.set_name("button2")
        self.button_2.set_label(_("upgrade directly"))
        self.button_2.connect("clicked", self.on_upgrade_click)
        
        screen = Gdk.Screen.get_default()
        self.window.set_size_request(600, 208)
        self.window.set_position(Gtk.WindowPosition.CENTER)
        self.window.connect("delete-event", self.destroy)
       

    def show(self):
        self.window.present()
        self.window.show_all()

    def destroy(self, dialog, event):
        self.window.destroy()
        
    def on_upgrade_click(self, button):
        self.window.destroy()
        update_system(False)
        reboot_dialog()

    def on_backup_upgrade_click(self, button):
        self.window.destroy()
        update_system(True)
        reboot_dialog()
        
