#!/usr/bin/python3
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk
from patch_lib import *
import time
import urllib.request, urllib.error, urllib.parse
import urllib.request, urllib.parse, urllib.error
import json
import os
import dbus
import dbus.service
import sys
from dbus.mainloop.glib import DBusGMainLoop
import threading
import traceback
import check_update
import logging
import _thread
import platform
import subprocess
import tempfile
import gettext
gettext.bindtextdomain('cdosupdate', "/usr/share/locale/")
gettext.textdomain('cdosupdate')
_ = gettext.gettext

global localhour
global flag 
global download_flag
global uidir
global timer_id

logging.basicConfig(format= '%(levelname)s %(asctime)-15s %(message)s file:%(filename)s line:%(lineno)d', filename=tempfile.mktemp("cdos-update-frontend.log"),level=logging.DEBUG)

timer_id = -1

installdir = "/usr/share/cdos-update/updates"
tmp_dir = "/tmp/cdosupdate"
uidir = sys.path[0] + "/ui"
download_flag = False

global dbus_object
dbus_object = None

global autoinstall, autodownload, willcheckupdate, dayfreq, hourfreq,check_last_time,install_last_time
autoinstall = False
autodownload = False
willcheckupdate = True
dayfreq = 0
hourfreq = 0
check_last_time = 0
install_last_time = 0

global getpatch_site_path
global getpatchlist_site_path

global mainCheckWindow
mainCheckWindow = None

class Status:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

# getpatchlisturl
def get_patchlist_url():
    global getpatch_site_path
    global getpatchlist_site_path
    global getpatchlistinfo_site_patch
    ip = "repos.01os.org"
    site_path = "https://" + ip + "/data/cdos-update/update/"
    getpatch_site_path = site_path + "getpatch/"
    getpatchlist_site_path = site_path + "getpatchlist"
    getpatchlistinfo_site_patch = site_path + "getpatchesinfoWithSystemVersion"
    return True

def get_local_package_list():
    # 获取本地已经正确安装的deb包及版本的json描述(即ii状态的deb包)
    deblist = subprocess.check_output(["dpkg","-l"]).split('\n')
    if len(deblist) <= 5:
        return Result(_("no deb list"), None)
    deb_desc_json = {}
    for i in range(5,len(deblist) - 1):
        deb_desc = deblist[i].split()
        if (len(deb_desc) > 0) and (deb_desc[0] == "ii"):
            deb_desc_json[deb_desc[1]] = deb_desc[2]
    return Result(None, deb_desc_json)

def check_package(patchinfo, local_package_info):
    try:
        # 如果没有包需要检测
        if ( (type(patchinfo) == dict) and (len(patchinfo) == 0) ) or (patchinfo == None) or (patchinfo == "null"):
            return Result(None, True)
        if type(patchinfo) != dict:
            return Result(_("Package version information type error"), None)
        # 开始校验包版本
        for deb_name in patchinfo:
            # 如果本地deb包中没有该包
            if deb_name not in local_package_info:
                return Result(_("Package version check failed"), None)
            # if type(patchinfo[deb_name]) != list:
            #     pass
            # 如果本地deb包版本不在patchinfo版本列表中
            if type(patchinfo[deb_name]) == list and len(patchinfo[deb_name]) != 0 and local_package_info[deb_name] not in patchinfo[deb_name]:
                return Result(_("Package version check failed"), None)
        return Result(None, True)
    except Exception as ee:
        logging.error('generic exception %s:  %s' % (ee, traceback.format_exc()))
        return Result(_("Package version check exception"), None)

def getRemotePatchlist():
    global getpatchlistinfo_site_patch
    url = getpatchlistinfo_site_patch
    data = {"systemversion":platform.dist()[1],"kernelversion":platform.uname()[2],"hardwareplatform":platform.machine()}
    try:
        request = urllib.request.Request(url,data = json.dumps(data))
        patchinfolist = json.load(urllib.request.urlopen(request, timeout=10) )
        remote_list = []
        # 没有本地deb列表，不更新
        local_package_info_res = get_local_package_list()
        if local_package_info_res.getError():
            return Result(None, [])
        local_package_info = local_package_info_res.getValue()
        for patchinfo in patchinfolist:
            check_res = check_package(patchinfo["package"], local_package_info)
            if check_res.getError():
                continue
            remote_list.append(patchinfo["id"])
        return Result(None, remote_list)
    except Exception as e:
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        return Result(_("Unknown exception, please check the network"), None)

class DialogUI():
    def __init__(self, patchlist, detail, enter_label, cancel_label, enter_callback, cancel_callback, cancel_hide=False):
        import gettext as gettext1
        gettext1.bindtextdomain('cdosupdate', '/usr/share/locale')
        gettext1.textdomain('cdosupdate')
        self._ = gettext1.gettext

        self.destroy_flag = False
        self.cancel_hide = cancel_hide

        self.builder = Gtk.Builder()
        self.builder.add_from_file(uidir + "/dialog.ui")
        
        self.window = self.builder.get_object("dialog-window")
        
        self.viewport = self.builder.get_object("dialog-viewport")
        self.viewport.set_name("dialog-viewport")

        self.title = self.builder.get_object('title')
        self.title.set_name("title")
        self.detail = self.builder.get_object("detail")
        self.detail.set_name("detail")
        
        self.enter_button = self.builder.get_object("enter")
        self.cancel_button = self.builder.get_object("cancel")
        self.enter_button.set_name("dia-enter")
        self.cancel_button.set_name("dia-cancel")
        self.enter_button.get_style_context().add_class("popup_patch_btn")
        self.cancel_button.get_style_context().add_class("popup_patch_btn")

        self.separator = self.builder.get_object("separator")
        self.separator.set_name("separator")

        self.title.set_markup(self._("NFS Desktop OS Update"))
        self.detail.set_markup(self._(detail))
        self.enter_button.set_label(self._(enter_label))
        self.cancel_button.set_label(self._(cancel_label))

        self.builder.connect_signals(self)
        if enter_callback != None:
            self.enter_button.connect("clicked", enter_callback, patchlist, self)
        if cancel_callback != None:
            self.cancel_button.connect("clicked", cancel_callback, patchlist, self)

        screen  = Gdk.Screen.get_default()
        css_provider = Gtk.CssProvider()
        css_provider.load_from_data('''
            #dialog-viewport{
                background: #FFF;
            }
            #separator{
                border:none;
                border-top:1px solid #DDD; 
            }
            .popup_patch_btn{
                border:none;
                color: #333;
                font-size: 13px;
                outline: none;
                /*box-shadow: 0 0 3px 2px #CCC;*/
                border-radius: 5px;
                padding: 0 10px;
            }
            #dia-enter{
                background: #97B6F4;
                color: #2C4986;
            }
            #dia-enter:hover{
                background: #97B6F4;
             }
            #dia-cancel{
                background: #CCC;
            }
            #title,
            #detail{
                font-size: 13px;
            }
            ''')
        context = Gtk.StyleContext()
        context.add_provider_for_screen(screen, css_provider,
          Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.window.set_size_request(300, 130)
        (self._width, self._height) = self.window.get_size()
        self.window.move(screen.get_width() - self._width, screen.get_height() - self._height)
        self.window.set_keep_above(True)

    def show(self):
        self.window.show_all()
        if self.cancel_hide:
            self.cancel_button.hide()

    def destroy(self):
        self.destroy_flag = True
        self.window.destroy()

global dialogui
def enter_call(obj, patchlist, ui):
    pass
def cancel_call(obj, patchlist, ui):
    pass
dialogui = DialogUI("", "", "", "", enter_call, cancel_call, cancel_hide = False)
dialogui.destroy()

def getLocalPatchlist():
    if dbus_object == None:
        return Result(str("Error: No DBUS service found"), None)
    try:
        installedlist = []
        tmppatchList = dbus_object.getInstalledList()
        for i in range(len(tmppatchList)):
            installedlist.append(str(tmppatchList[i]))
        downloadedlist = []
        tmppatchList = dbus_object.getDownloadedList()
        for i in range(len(tmppatchList)):
            downloadedlist.append(str(tmppatchList[i]))
        patchlist = {}
        alllist = []
        # 整合补丁列表
        for patch in installedlist:
            if patch not in alllist:
                alllist.append(patch)
        for patch in downloadedlist:
            if patch not in alllist:
                alllist.append(patch)
        patchlist["installed"] = installedlist
        patchlist["downloaded"] = downloadedlist
        patchlist["all"] = alllist
        logging.debug(str(patchlist))
        return Result(None, patchlist)
    except Exception as e:
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        return Result(str(e), None)

# 第一个参数为本地的补丁列表，第二个参数为从服务器获取的补丁列表
def compareTwoList(list1, list2):
    try:
        toDOList = []
        for member in list2:
            if member not in list1:    # 如果远程补丁列表中有补丁不在本地
                toDOList.append(member)
        return Result(None, toDOList)
    except Exception as e:
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        return Result(str(e), None)

def download_enter_call(obj, patchlist, ui):
    global getpatch_site_path
    dbus_object.install(getpatch_site_path, patchlist)
    ui.destroy_flag = True
    ui.destroy()

def download_cancel_call(obj, patchlist, ui):
    ui.destroy_flag = True
    ui.destroy()

def install_enter_call(obj, patchlist, ui):
    global getpatch_site_path
    dbus_object.install(getpatch_site_path, patchlist)
    ui.destroy_flag = True
    ui.destroy()

def install_cancel_call(obj, patchlist, ui):
    ui.destroy_flag = True
    ui.destroy()

def installed_enter_call(obj, patchlist, ui):
    ui.destroy_flag = True
    ui.destroy()

def reboot_enter_call(obj, patchlist, ui):
    ui.destroy_flag = True
    ui.destroy()
    try:
        reboot_bus = dbus.SystemBus()
        reboot_dbus_obj = reboot_bus.get_object("org.freedesktop.ConsoleKit", "/org/freedesktop/ConsoleKit/Manager")
        reboot_dbus_iface = dbus.Interface(reboot_dbus_obj, "org.freedesktop.ConsoleKit.Manager")
        restart_res = reboot_dbus_iface.Restart()
        # print restart_res
        # print reboot_dbus_iface.getProperties()
    except Exception as e:
        logging.warning("reboot exception: %s" % str(e))

def reboot_cancel_call(obj, patchlist, ui):
    ui.destroy_flag = True
    ui.destroy()

def refreshUpdateThread(allPatchlist,isMainWindow=False):
    logging.debug("refreshUpdateThread isMainWindow=%s" % isMainWindow)
    global dialogui
    global getpatch_site_path
    # 获取本地与远程列表成功后，设置checktime
    setLastCheckTime()
    if isMainWindow == False and willcheckupdate == False:
        return False
    if  not  allPatchlist:
        if isMainWindow:
            mainCheckWindow.showUpdated(refreshUpdate)
        return False
    get_local_patch_list_res = getLocalPatchlist()
    if get_local_patch_list_res.getError() != None:
        logging.error(get_local_patch_list_res.getError())
        mainCheckWindow.showCheckError(refreshUpdate)
        return False

    # 比较补丁列表
    localPatchList = get_local_patch_list_res.getValue()

    if autoinstall == True:
        compare_res = compareTwoList(localPatchList["installed"], allPatchlist)
        if compare_res.getError() != None:
            logging.error(compare_res.getError())
            mainCheckWindow.showCheckError(refreshUpdate)
            return False
        else:
            toDOList = compare_res.getValue()
            if len(toDOList) > 0:
                dbus_object.install(getpatch_site_path, toDOList)
            else:
                if isMainWindow:
                    mainCheckWindow.showUpdated(refreshUpdate)
            return False
    elif autodownload == True:
        compare_res = compareTwoList(localPatchList["installed"], allPatchlist)
        if compare_res.getError() != None:
            logging.error(compare_res.getError())
            mainCheckWindow.showCheckError(refreshUpdate)
            return False
        else:
            toDOList = compare_res.getValue()
            if len(toDOList) > 0:
                compare_res = compareTwoList(localPatchList["all"], allPatchlist)
                if compare_res.getError() != None:
                    logging.error(compare_res.getError())
                    mainCheckWindow.showCheckError(refreshUpdate)
                    return False
                else:
                    toDownloadList = compare_res.getValue()
                    if len(toDownloadList) > 0:
                        dbus_object.download(getpatch_site_path, toDownloadList)
                    else:
                        if isMainWindow:
                            def installCallback(installList):
                                dbus_object.install(getpatch_site_path, installList)
                            mainCheckWindow.showToInstallAfterDownload(toDOList, installCallback)
                        else:
                            if dialogui.destroy_flag == False:
                                dialogui.destroy()
                            dialogui = DialogUI(toDOList, "Install downloaded updates?", "yes", "no", install_enter_call, install_cancel_call, cancel_hide = False)
                            dialogui.show()
                    return False
            else:
                if isMainWindow:
                    mainCheckWindow.showUpdated(refreshUpdate)
            return False
    else:
        compare_res = compareTwoList(localPatchList["installed"], allPatchlist)
        if compare_res.getError() != None:
            logging.error(compare_res.getError())
            mainCheckWindow.showCheckError(refreshUpdate)
            return False
        else:
            toDOList = compare_res.getValue()
            if len(toDOList) > 0:
                if isMainWindow:
                    def installCallback(installList):
                        dbus_object.install(getpatch_site_path, installList)
                    mainCheckWindow.showToInstall(toDOList, installCallback)
                else:
                    if dialogui.destroy_flag == False:
                        dialogui.destroy()
                    dialogui = DialogUI(toDOList, "Download and install updates?", "yes", "no", install_enter_call, install_cancel_call, cancel_hide = False)
                    dialogui.show()
            else:
                if isMainWindow:
                    mainCheckWindow.showUpdated(refreshUpdate)
            return False


def getRemotPatch(callback,isMainWindow):
    patchlist_res = getRemotePatchlist()
    if patchlist_res.getError() != None:
        logging.error(patchlist_res.getError())
        mainCheckWindow.showCheckError(refreshUpdate)
        patchlist = []
    else:
        patchlist = patchlist_res.getValue()
        GLib.idle_add(callback,patchlist,isMainWindow)

def refreshUpdate(isMainWindow=False):
    if isMainWindow:
        get_patchlist_url()
    _thread.start_new_thread(getRemotPatch,(refreshUpdateThread, isMainWindow))

def update_timer():
    global timer_id
    logging.debug('update_timer')
    
    determine_res = determineCheck()
    if timer_id > -1:
        GLib.source_remove(timer_id)
        timer_id = -1
    # 出错
    if determine_res.getError() != None:
        timer_id = GLib.timeout_add_seconds(60, update_timer)
        return Result(determine_res.getError(), None)
    # 不需要进行补丁升级检查
    if determine_res.getValue() == False:
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
        return Result("it's not time for patch", None)
    # 需要进行补丁升级检查
    else:
        # 检查是否需要更新
        mainCheckWindow.set_last_checkTime(check_last_time)
        refreshUpdate()
        getsleeptime_res = getSleeptimeBeforeNextCheck()
        if getsleeptime_res.getError() != None:
            timer_id = GLib.timeout_add_seconds(60, update_timer)
        else:
            timer_id = GLib.timeout_add_seconds(getsleeptime_res.getValue(), update_timer)
    return 1

def popup_menu(icon, button, time):
    mainCheckWindow.show()

def activate_icon_cb(icon):
    mainCheckWindow.show()

def handle_statusChanged(statusCode, status_text, patchID, correct_install_patches_list, patchList, progress, paramList):
    global dialogui
    global getpatch_site_path
    logging.debug(str(statusCode) + " " + status_text + " " + patchID + " " + str(correct_install_patches_list) + " " + str(patchList) + " " + str(progress))
    if statusCode == Status.DOWNLOADING:
        mainCheckWindow.showDownloading(status_text, progress)
    elif statusCode == Status.DOWNLOADED:
        mainCheckWindow.showDownloading(status_text, progress)
    elif statusCode == Status.ALLDOWNLOADED:
        def installCallback(installList):
            dbus_object.install(getpatch_site_path, installList)
        mainCheckWindow.showToInstallAfterDownload(patchList, installCallback)
        if autodownload:
            if dialogui.destroy_flag == False:
                dialogui.destroy()
            dialogui = DialogUI(patchList, "Install downloaded updates?", "yes", "no", install_enter_call, install_cancel_call, cancel_hide = False)
            dialogui.show()
    elif statusCode == Status.UPDATING:
        mainCheckWindow.showUpdating(status_text, progress)
    elif statusCode == Status.VERIFYING:
        mainCheckWindow.showUpdating(status_text, progress)
    elif statusCode == Status.INSTALLING:
        mainCheckWindow.showUpdating(status_text, progress)
    elif statusCode == Status.INSTALLED:
        setLastInstallTime()
        mainCheckWindow.showUpdating(status_text, progress)
    elif statusCode == Status.ALLINSTALLED:
        un_install_patch_list_res = compareTwoList(correct_install_patches_list, patchList)
        if un_install_patch_list_res.getError() != None:
            logging.error(un_install_patch_list_res.getError())
            un_install_patch_list = []
        else:
            un_install_patch_list = un_install_patch_list_res.getValue()
        mainCheckWindow.showAllInstalled(un_install_patch_list, refreshUpdate)
        if dialogui.destroy_flag == False:
            dialogui.destroy()
        setLastInstallTime()
        if ('reboot' in paramList) and (paramList['reboot'] == True):
            dialogui = DialogUI(patchList, "Updates install succeed,some\npatches need be reboot to be\neffective,reboot now?", "yes", "no", reboot_enter_call, reboot_cancel_call, cancel_hide = False)
            dialogui.show()
        else:
            dialogui = DialogUI(patchList, "Updates install succeed", "ok", "", installed_enter_call, None, cancel_hide = True)
            dialogui.show()
    elif statusCode == Status.ERROR_INSTALL:
        mainCheckWindow.showError(status_text, refreshUpdate)
        if dialogui.destroy_flag == False:
            dialogui.destroy()
        dialogui = DialogUI(patchList, "Updates install failed", "ok", "", installed_enter_call, None, cancel_hide = True)
        dialogui.show()
    else:
        pass

def main_parseArgs(argv):
    logging.debug("main_parseArgs:" + str(argv))
    if len(argv) > 1:
        if argv[1] == "show":
            mainCheckWindow.show()
 
global intern_dbus_name
global intern_dbus_path
global intern_dbus_iface
intern_dbus_name = "org.cdos.updateClient"
intern_dbus_path = "/org/cdos/updateClient"
intern_dbus_iface =  "org.cdos.updateClient"

class OneInstance(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(intern_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, intern_dbus_path)

    @dbus.service.method(intern_dbus_iface, in_signature='as', out_signature='b')
    def main(self, argv):
        argList = []
        for i in range(len(argv)):
            argList.append(str(argv[i]))
        if len(argv) == 1:
            argList.append("show")
        main_parseArgs(argList)
        return True

if __name__ == "__main__":

    if not get_patchlist_url():
        logging.error("Error: no settings file found.")
        sys.exit(1)  #进程退出

    try:
        dbml = DBusGMainLoop(set_as_default=True)
        bus = dbus.SystemBus()
        remote_object = bus.get_object("org.cdos.update", "/org/cdos/update")
        dbus_object = dbus.Interface(remote_object, 'org.cdos.update')

        bus.add_signal_receiver(handle_statusChanged,'statusChanged', "org.cdos.update", "org.cdos.update", "/org/cdos/update")
        dbus.set_default_main_loop(dbml)
    except Exception as e:
        logging.error("Error Happened, please check the dbus service org.cdos.update has started")
        logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
        sys.exit(1)  #进程退出

    try:
        bus = dbus.SessionBus()
        remote_object_2 = bus.get_object(intern_dbus_name, intern_dbus_path )
        iface = dbus.Interface(remote_object_2, intern_dbus_iface)
        iface.main(sys.argv)
        sys.exit(0)
    except Exception as e:
        oneinstance = OneInstance()

    schema = Gio.Settings(schema = "org.cdos.update")
    def on_install_changed(self,c):
        global autoinstall
        autoinstall = schema.get_boolean("autoinstall")

    def on_download_changed(self, c):
        global autodownload
        autodownload = schema.get_boolean("autodownload")

    def on_willcheckupdate_changed(self, c):
        global willcheckupdate
        willcheckupdate = schema.get_boolean("checkupdate")
 
    def on_check_last_time(self,c):
        global check_last_time
        check_last_time = schema.get_int("last-checkupdate-time")
        mainCheckWindow.set_last_checkTime(check_last_time)
        
    def on_install_last_time(self,c):
        global  install_last_time
        install_last_time = schema.get_int("last-install-time")
        mainCheckWindow.set_last_installTime(install_last_time)
        
    autoinstall = schema.get_boolean("autoinstall")
    autodownload = schema.get_boolean("autodownload")
    willcheckupdate = schema.get_boolean("checkupdate")
    check_last_time = schema.get_int("last-checkupdate-time")
    install_last_time = schema.get_int("last-install-time")
    schema.connect("changed::autoinstall", on_install_changed)
    schema.connect("changed::autodownload", on_download_changed)
    schema.connect("changed::checkupdate", on_willcheckupdate_changed)
    schema.connect("changed::last-checkupdate-time", on_check_last_time)
    schema.connect("changed::last-install-time", on_install_last_time)

    def on_dayfreq_changed(c):
        global dayfreq
        global timer_id
        dayfreq = schema.get_int("check-day")
        mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_dayfreq_changed(self, c):
        GLib.idle_add(on_dayfreq_changed, c)

    def on_hourfreq_changed(c):
        global hourfreq
        global timer_id
        hourfreq = schema.get_int("check-hour")
        mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
        if timer_id and timer_id > -1:
            GLib.source_remove(timer_id)
            timer_id = -1
        timer_id = GLib.timeout_add_seconds(0, update_timer)
        return False

    def idle_on_hourfreq_changed(self, c):
        GLib.idle_add(on_hourfreq_changed, c)

    dayfreq = schema.get_int("check-day")
    hourfreq = schema.get_int("check-hour")
    schema.connect("changed::check-day", idle_on_dayfreq_changed)
    schema.connect("changed::check-hour", idle_on_hourfreq_changed)

    mainCheckWindow = check_update.UICheck(uidir)

    mainCheckWindow.set_last_checkTime(check_last_time)
    mainCheckWindow.set_last_installTime(install_last_time)
    mainCheckWindow.setDayFrequecy(dayfreq, hourfreq)
    mainCheckWindow.setFrequecy(willcheckupdate, autodownload, autoinstall)

    mainCheckWindow.showInitPage(refreshUpdate)

    main_parseArgs(sys.argv)

    timer_id = GLib.timeout_add_seconds(3, update_timer)
    Gtk.main()
