# -*- coding:utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sqlite3
import sys
try:
    reload(sys)
    sys.setdefaultencoding('utf8')
except Exception as e:
    pass
import os
import logging

class history_db():
    def __init__(self, history_path=""):
        if history_path == "":
            self.history_path = sys.path[0]+'/.history/history.db'
        else:
            self.history_path = history_path
        self.create()

    def create(self):
        table_new = """CREATE TABLE history (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        type       varchar(20) NOT NULL,
                        appname    varchar(200) DEFAULT NULL,
                        size       varchar(20) DEFAULT NULL,
                        version    varchar(50) DEFAULT NULL,
                        updateTime bigint(20) NOT NULL,
                        icon       varchar(1024) DEFAULT NULL,
                        shortdesc  varchar(255) DEFAULT NULL,
                        longdesc   longtext DEFAULT NULL)"""
        if os.path.exists(self.history_path):
            return

        conn = None
        try:
            dir_name = os.path.dirname(self.history_path)
            if os.path.isfile(dir_name):
                os.remove(dir_name)
            if not os.path.isdir(dir_name):
                os.makedirs(dir_name)
            conn = sqlite3.connect(self.history_path)
            cur = conn.cursor()
            cur.execute(table_new)
            conn.commit()
        except Exception as e:
            logging.error( "create history database " + str(e) )
        finally:
            if conn is not None:
                conn.close()

    def search(self,days):
        ret = []
        conn = None
        try:
            command = "SELECT * FROM history WHERE JULIANDAY('now') - %d <= JULIANDAY( updateTime,'unixepoch') " % days
            conn = sqlite3.connect(self.history_path)
            cur = conn.cursor()
            itr = cur.execute(command)
            for value in itr:
                onerecord = {}
                onerecord['type'] = value[1]
                onerecord['appname'] = value[2]
                onerecord['version'] = value[3]
                onerecord['size'] = value[4]
                onerecord['updateTime'] = value[5]
                onerecord['icon'] = value[6]
                onerecord['shortdesc'] = value[7]
                onerecord['longdesc'] = value[8]
                ret.append(onerecord)
        except Exception as e:
            logging.error( "search history database " + str(e) )
        finally:
            if conn is not None:
                conn.close()
        return ret

    def insert(self, update_type, updateTime, appname='', version='', size='', icon='', shortdesc='', longdesc=''):
        try:
            command = "INSERT INTO history (type, appname, version, size, updateTime, icon, shortdesc, longdesc) "\
                    "VALUES ('%s', '%s', '%s', '%s', %d, '%s', '%s', '%s')" % ( update_type, appname, version, size, updateTime, icon, shortdesc, longdesc )
            conn = sqlite3.connect(self.history_path)
            cur = conn.cursor()
            cur.execute(command)
            conn.commit()        
        except Exception as e:
            logging.error( "insert history database " + str(e) )
        finally:
            if conn is not None:
                conn.close()  

if __name__ == '__main__':
    import time
    import datetime

    # db = history_db()
    # db.create()

    home_dir = os.getenv("HOME")
    history_db = history_db(os.path.join(home_dir, "config/cdos-update/history.db"))

    # # 插入
    # cnt = 0
    # while cnt < 5:
    #     db.insert(type='app', appname='code'+str(cnt), updateTime=int(time.time()), longdesc='hello')
    #     time.sleep(1)
    #     cnt += 1
    
    # 最近10天
    for info in history_db.search(10):
        print(info)
