#!/usr/bin/python3
#-*-coding:UTF-8-*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

from .lib_result import *
from gi.repository import Gio
import datetime
import logging

# 设置上次检查更新的时间
def setLastCheckTime():
    try:
        cur_datetime_res = timeStamp(datetime.datetime.now())
        if cur_datetime_res.getError() != None:
            return Result(cur_datetime_res.getError(), None)
        Gio.Settings(schema = "org.cdos.update").set_int("last-checkupdate-time", int(cur_datetime_res.getValue()) )
        return Result(None, True)
    except Exception as e:
        return Result(str(e), False)

# 获取上次检查更新的时间
def getLastCheckTime():
    try:
        schema = Gio.Settings(schema = "org.cdos.update")
        lastCheckTime = schema.get_int("last-checkupdate-time")
        return Result(None, lastCheckTime)
    except Exception as e:
        return Result(str(e), False)

# 设置上次安装更新的时间
def setLastInstallTime():
    try:
        cur_datetime_res = timeStamp(datetime.datetime.now())
        if cur_datetime_res.getError() != None:
            return Result(cur_datetime_res.getError(), None)
        Gio.Settings(schema = "org.cdos.update").set_int("last-install-time", int(cur_datetime_res.getValue()) )
        return Result(None, True)
    except Exception as e:
        return Result(str(e), False)

# 获取上次安装更新的时间
def getLastInstallTime():
    try:
        lastInstallTime = Gio.Settings(schema = "org.cdos.update").get_int("last-install-time")
        return Result(None, lastInstallTime)
    except Exception as e:
        return Result(str(e), False)

# 获取时间戳
def timeStamp(new_datetime):
    try:
        dateArray = datetime.datetime.fromtimestamp(0)
        timestamp = (new_datetime - dateArray).total_seconds()
        return Result(None, timestamp)
    except Exception as e:
        return Result(str(e), None)

# 获取根据用户配置，本地时间之前应该检测的时间
def getLastShouldCheckTime():
    try:
        schema = Gio.Settings(schema = "org.cdos.update")
        check_weekday = schema.get_int("check-day")
        check_hour = schema.get_int("check-hour")
        if check_weekday == 0:
            check_weekday = 7
        else:
            check_weekday = check_weekday-1

        cur_time = datetime.datetime.now()
        cur_year = cur_time.year
        cur_mon = cur_time.month
        cur_day = cur_time.day
        cur_hour = cur_time.hour
        cur_weekday = cur_time.weekday()
        checked_hours = 0

        if check_weekday != 7:
            # 今天在检测时间weenkday之后
            if (cur_weekday > check_weekday) or (cur_weekday == check_weekday and cur_hour >= check_hour): 
                checked_hours = (cur_weekday - check_weekday)*24 + (cur_hour - check_hour)
            # 今天在检测时间weenkday之前
            else:
                checked_hours = (cur_weekday + 7 - check_weekday)*24 + (cur_hour - check_hour)
        else:  # 每天
            if cur_hour >= check_hour:
                checked_hours = (cur_hour - check_hour)
            else:
                checked_hours = (cur_hour + 24 - check_hour)
        cur_datetime = datetime.datetime(year = cur_year, month = cur_mon, day = cur_day, hour = cur_hour)
        last_datetime = cur_datetime - datetime.timedelta(hours = checked_hours)
        lastShouldCheckTimeres = timeStamp(last_datetime)
        if lastShouldCheckTimeres.getError() != None:
            return Result(lastShouldCheckTimeres.getError(), None)
        else:
            return Result(None, lastShouldCheckTimeres.getValue())
    except Exception as e:
        return Result(str(e), None)

# 获取根据用户配置，本地时间之后应该检测的时间
def getNextShouldCheckTime():
    try:
        schema = Gio.Settings(schema = "org.cdos.update")
        check_weekday = schema.get_int("check-day")
        if check_weekday == 0:
            check_weekday = 7
        else:
            check_weekday = check_weekday-1

        getLastShouldCheckTimeres = getLastShouldCheckTime()
        if getLastShouldCheckTimeres.getError() != None:
            return Result(getLastShouldCheckTimeres.getError(), None)
        else:
            lastShouldCheckTime = getLastShouldCheckTimeres.getValue()
            if check_weekday != 7:
                nextShouldCheckTime = lastShouldCheckTime + 7*24*3600
            else:  # 每天
                nextShouldCheckTime = lastShouldCheckTime + 24*3600
            return Result(None, nextShouldCheckTime)
    except Exception as e:
        return Result(str(e), None)

# 决定是否检查
def determineCheck():
    try:
        last_should_time_res = getLastShouldCheckTime()
        if last_should_time_res.getError() != None:
            return Result(last_should_time_res.getError(), None)
        last_should_time = last_should_time_res.getValue()

        last_check_time_res = getLastCheckTime()
        if last_check_time_res.getError() != None:
            return Result(last_check_time_res.getError(), None)
        last_check_time = last_check_time_res.getValue()
        logging.debug(last_should_time)
        logging.debug(last_check_time)

        if last_check_time >= last_should_time:
            logging.debug("determineCheck return false")
            return Result(None, False)
        else:
            logging.debug("determineCheck return true")
            return Result(None, True)
    except Exception as e:
        return Result(str(e), None)

# 获取下次检测之前睡眠时间
def getSleeptimeBeforeNextCheck():
    try:
        get_next_should_time_res = getNextShouldCheckTime()
        if get_next_should_time_res.getError() != None:
            return Result(get_next_should_time_res.getError(), None)
        next_time = get_next_should_time_res.getValue()
        cur_datetime_res = timeStamp(datetime.datetime.now())
        if cur_datetime_res.getError() != None:
            return Result(cur_datetime_res.getError(), None)
        sleep_time = int(next_time - cur_datetime_res.getValue())
        if sleep_time == 0:
            sleep_time = sleep_time + 1
        logging.debug("sleep_time %i " % sleep_time)
        return Result(None, sleep_time)
    except Exception as e:
        return Result(str(e), None)
