#!/usr/bin/python3
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from gi.repository.GdkPixbuf import Pixbuf,PixbufAnimation

class WarningDialog(object):

    user_choice = "cancel"

    def __init__(self, parent, uidir, imgdir, closewin=False, cancel_button=True, confirm_button=False, callback=None):
        import gettext as gettext1
        gettext1.bindtextdomain('cdosupdate', "/usr/share/locale/")
        gettext1.textdomain('cdosupdate')
        self._ = gettext1.gettext

        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\
            #warning_window {
                background: #FCFCFC;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
        self.callback = callback
        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.join(uidir, "warning.ui"))
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("warning_window")
        self.window.set_name("warning_window")
        self.window.set_title(self._("Warning"))

        self.image = self.builder.get_object("warning_img")
        self.warning_label = self.builder.get_object("warning_label")
        self.warning_label.set_name("warning_label")

        self.cancel_button = self.builder.get_object("cancel")
        self.cancel_button.set_name("cancel")
        self.cancel_button.set_label(self._("Close"))
        if closewin:
            self.cancel_button.connect("clicked", self.do_close)
        else:
            self.cancel_button.connect("clicked", self.on_cancel)
        self.cancel_button.set_size_request( 80, 30)
        if cancel_button:
            pass
        else:
            self.cancel_button.hide()

        self.confirm_button = self.builder.get_object("confirm")
        self.confirm_button.set_name("confirm")
        self.confirm_button.set_label(self._("Confirm"))
        if confirm_button:
            self.cancel_button.set_label(self._("Cancel"))
            self.confirm_button.connect("clicked", self.do_confirm)
            self.confirm_button.set_size_request( 80, 30)
        else:
            self.confirm_button.hide()

        image_name = os.path.join(os.path.join(imgdir, "warning.png"))
        pixbuf = Pixbuf.new_from_file_at_scale(image_name, width=48, height=48, preserve_aspect_ratio=False)
        self.image.set_from_pixbuf(pixbuf)
        if closewin:
            self.window.connect("delete-event", self.do_close)
        self.window.set_keep_above(True)
        self.window.set_position(Gtk.WindowPosition.CENTER)
        # self.parent = parent

    def show(self):
        self.window.show()

    def set_label(self, label_info):
        self.warning_label.set_text(label_info)

    def set_locale_label(self, locale_label_info, *args):
        self.warning_label.set_text(self._(locale_label_info) %args)

    def on_cancel(self, widget):
        self.user_choice = "cancel"
        self.window.destroy()
        self.callback(self.user_choice)

    def do_close(self, widget, event=None):
        Gtk.main_quit()

    def do_confirm(self, widget):
        self.user_choice = "confirm"
        self.window.destroy()
        self.callback(self.user_choice)

# if __name__ == "__main__":
#     warning_dialog = WarningDialog(os.getcwd())

#     warning_dialog.show()

#     Gtk.main()



